/**
 *           Advanced Real-Time Simulation Lab (ARS) - Carleton University
 *
 * PROJECT: 			Web Service-Enabled DCD++ Client
 * AUTHOR(S): 			Rami Madhoun
 * PURPOSE: 			Euaclyptus
 * CMD LINE ARG:
 * OUTPUT:
 * DATE CREATED: 		May 1, 2006
 * LAST MOD:   			Jan 1, 2006
 * LAST MOD BY:			Rami Madhoun
 * COPYRIGHT:			ARS-Carleton University
 * USAGE:			The wrapper proxy is an interface bwteen the web service components and 
 *                              the simulation components. It is responsible for passing messages between 
 * 				the two components for all the sessions
 * COMMENTS:
 **/



#include "wrapperproxy.h"
#include <iostream>
#include "fstream"
#include <string>
#include <map>
#include <sys/ipc.h>
#include <sys/msg.h>
#include <pthread.h>
#include <stdlib.h>
#include <sstream>

#include <time.h>

#include <sys/time.h>

using namespace std;

enum {
	WRAPPER_ORIGINATED	= 1,
	CDPP_ORIGINATED		= 2
     } ;

// Message types
enum {  
	CURRENT_SIMULATION_TIME_REQUEST 	= 1 , // The client will use it to query the simulation time
	CURRENT_SIMULATION_TIME_RESPONSE 	= 2 , // The current simulation time response
	INSERT_EXTERNAL_EVENT_REQUEST    	= 3 , // Used to insert external events while the simulation 
						      // is running
	MACHINE_FOR_MODEL_REQUEST 		= 4 , // Used by the simulation components to check the model 
						      // partition
	MACHINE_FOR_MODEL_RESPONSE		= 5 , // Used by the web service components to sent the model
						      // partition to the simulation components
	SIMULATION_STOP				= 6 , // Used to stop the simulation
	GET_MACHINE_ID_REQUEST			= 7 , // Checks the machine id (sent by the simulation comps.)
	GET_MACHINE_ID_RESPONSE			= 8 , // Responds with the machine is (sent by the WS comps.)
	INIT_MESSAGE 				= 9 , // Initialization I message
	DONE_MESSAGE			        = 10, // Done D message
	COLLECT_MESSAGE		             	= 11, // Collect @ message
	INTERNAL_MESSAGE	             	= 12, // Internal * message
	EXTERNAL_MESSAGE	             	= 13, // External X message
	OUTPUT_MESSAGE		            	= 14, // Output Y message
	ADD_ZONE_PARTITION			= 15, // Used to add the zones of Cell-DEVS models 
						      // (sent by the simulation comps.)
	ECHO_MESSAGE				= 16  // Used to testing the performance of the simulator
	} ;

string cdppParentPath = "";
map<int,int, less<int> > sessions_IPCSendQueues;     //  message queues to send messages
map<int,int, less<int> > sessions_IPCReceiveQueues;  // message queues to receive messages
map<int,pthread_t, less<int> > sessions_MessageMonitors;  // map of session ids and the corresponding message 
							  // queue monitors
//map<int, JNIEnv*, less<int> > sessions_JEnvs;

//JavaVM* jvm;

ofstream d_file;

int open_queue(key_t);
int read_message(int, long ,struct messageBuffer*);
int send_message(int, struct messageBuffer*);
int remove_queue( int );

void messageMonitor(int );

string waitForCurrentSimulationTime(int); 
void machineForModel(int, char*, JNIEnv* env); 
void getMachineID(int, JNIEnv *env) ;
void deleteQueues(int);
void sendRemoteMessage(messageBuffer*, JNIEnv* env);

static jclass jni_class = NULL;
static jmethodID machineForModel_methodId = NULL;
static jmethodID getMachineID_methodId = NULL;


// Structure to transfer the different messages between the w.s. components and simulation components
struct messageBuffer {
        long mtype;         // type of the message : CDPP_ORIGINATED/ WRAPPER_ORIGINATED
        long specific_type; // specific type message ( as defined above )
        int session_id;     // session id
        long longParam1;    // process id as in msg.ProcId
        long longParam2;    // port id as in msg.Port().id()
        long longParam3;    // msg.SenderModelId()
        long longParam4;    // machine id 
        long longParam5;    // process id

	long longParam6;    // time (seconds)
	long longParam7;    // time (u. seconds)

        bool boolParam1;      // msg.fromSlave()
        double doubleParam1;  // msg.value()
        char charParam1[80];  // msg.time().asString()
        char charParam2[80];  // msg.nextChange()
        char charParam3[80];  // msg.senderCellPosition()
        } ;



// Sets the cdpp path which is used to locate the file used to initialize the message queues

JNIEXPORT void JNICALL Java_SimulationService_wrapper_JavaWrapper_setCDppParentPath(JNIEnv * env , jclass cls, jstring path_) 
{
        const char* buff_p;
        char buffer[128];
        
        buff_p = env->GetStringUTFChars(path_,0);

        strcpy(buffer,buff_p);

	cdppParentPath = buffer;
	env->ReleaseStringUTFChars(path_,buff_p);
		
}

// Initializes new session by establishing the send and receive message queues associated with this session
JNIEXPORT void JNICALL Java_SimulationService_wrapper_JavaWrapper_initializeNewSession
  (JNIEnv * env, jclass cls , jint session_id_) 
{

	stringstream ss;
	string tempStr ;
	ss << session_id_ ;
	ss >> tempStr;
	string cdppPath_ = cdppParentPath + "/" + tempStr + "/" + "ftok.dum";

	int session_id = session_id_;
	int send_queue_id, receive_queue_id;

	key_t send_queue_key = ftok(cdppPath_.c_str(),'s');
	key_t receive_queue_key = ftok(cdppPath_.c_str(),'r');

	send_queue_id = open_queue(send_queue_key);
	receive_queue_id = open_queue(receive_queue_key);

	pair<int,int> p1 (session_id,send_queue_id);
	pair<int,int> p2 (session_id,receive_queue_id);

	sessions_IPCSendQueues.insert(p1);
	sessions_IPCReceiveQueues.insert(p2);
}

// Sends a request to the simulation components to retrieve the current simulation time
JNIEXPORT jstring JNICALL Java_SimulationService_wrapper_JavaWrapper_getCurrentSimulationTime
( JNIEnv * env, jobject obj, jint sessionID_)
{
	int result = -1;
	map<int,int>::const_iterator cursor;
	cursor = sessions_IPCSendQueues.find((int) sessionID_);
	int send_queue_id = cursor->second;
	messageBuffer msg ;
	msg.mtype = WRAPPER_ORIGINATED;
	msg.specific_type = CURRENT_SIMULATION_TIME_REQUEST;
	msg.session_id = (int) sessionID_;
	result = send_message(send_queue_id, &msg );

	string simTime_ = waitForCurrentSimulationTime( (int) sessionID_);
	return env->NewStringUTF(simTime_.c_str());

}

// Waits for the response from the simulation components to the simulation time request
string waitForCurrentSimulationTime (int sessionID_)
{
	int receive_queue_id , result ;
	string simTime_  ;
	map<int,int>::const_iterator cursor ;
	cursor = sessions_IPCReceiveQueues.find((int) sessionID_);
	receive_queue_id = cursor->second;
	cout << "receive_queue_is : " << receive_queue_id << endl;

	messageBuffer msg ;
	
	result = read_message(receive_queue_id, WRAPPER_ORIGINATED , &msg );

	if (result != -1)	
		if(msg.specific_type == CURRENT_SIMULATION_TIME_RESPONSE)
			simTime_ = string(msg.charParam1);

	return simTime_;
}


// Inserts an external event while the simulation is running
JNIEXPORT void JNICALL Java_SimulationService_wrapper_JavaWrapper_insertExternalEvent
( JNIEnv *env, jobject obj, int sessionID_, jstring time_, jstring port,jdouble value)
{
        string event_time ,event_port;
        double event_value;

        const char* buff_p;
        char buffer[128];

        buff_p = env->GetStringUTFChars(time_,0);
        strcpy(buffer, buff_p);
        event_time = buffer;

	env->ReleaseStringUTFChars(time_,buff_p);

        buff_p = env->GetStringUTFChars(port,0);
        strcpy(buffer,buff_p);
        event_port = buffer;
		env->ReleaseStringUTFChars(port, buff_p);

        event_value = value;

        map<int,int>::const_iterator cursor;
        cursor = sessions_IPCSendQueues.find((int) sessionID_);
        int send_queue_id = cursor->second;

        messageBuffer msg;
    	msg.mtype = WRAPPER_ORIGINATED;
        msg.specific_type = INSERT_EXTERNAL_EVENT_REQUEST;
        msg.session_id = (int) sessionID_;
        msg.doubleParam1 = event_value;
        strcpy(msg.charParam1, event_time.c_str());
        strcpy(msg.charParam2, event_port.c_str());

        int result = send_message(send_queue_id, &msg);
}


// Starts the message monitoring function to check for incoming messages from the simulation components
JNIEXPORT void JNICALL Java_SimulationService_wrapper_JavaWrapper_startMessageMonitor  (JNIEnv * env, jobject obj, jint session_id) 
{
	map<int,int>::const_iterator cursor;
	cursor = sessions_IPCReceiveQueues.find(session_id);

	if(cursor != sessions_IPCReceiveQueues.end() )
	{
		int queue_id = cursor->second;
        	messageBuffer msg;
	        int result = -1;
        	bool stopSimulation= false ;

	        while( !stopSimulation )
        	{
                	if((result = read_message(queue_id, CDPP_ORIGINATED,&msg)) != -1)
	                {
        	                switch(msg.specific_type) {
                	                case MACHINE_FOR_MODEL_REQUEST :
                                	        machineForModel(msg.session_id, msg.charParam1, env);
                                        	break;
	                                case SIMULATION_STOP :
                	                        deleteQueues(msg.session_id);
                                        	stopSimulation = true;
	                                        break;
        	                        case GET_MACHINE_ID_REQUEST :
                        	                getMachineID(msg.session_id, env);
                                	        break;
	                                case INIT_MESSAGE:
        	                        case DONE_MESSAGE:
                	                case COLLECT_MESSAGE:
                        	        case INTERNAL_MESSAGE:
                                	case EXTERNAL_MESSAGE:
                               		case OUTPUT_MESSAGE:
					case ECHO_MESSAGE:
	                                        sendRemoteMessage(&msg, env);
        	                                break;
                	                default :
                        	                d_file << "message monitor got an unidentified message type " << endl << flush;
				}                                
			}

		}
	}
}

// Opens a message queue
int open_queue(key_t keyVal)
{
	int qid;
	if((qid = msgget(keyVal, IPC_CREAT | 0660 )) == -1)
	{
		return -1;
	}
	return qid;
}

// Reads a message from a message queue
int read_message(int qid,long msg_type, struct messageBuffer* qbuf)
{
	int result,length ;
	
	length = sizeof(struct messageBuffer) -sizeof(long);

	if((result = msgrcv(qid, qbuf , length , msg_type , 0 )) == -1)
	{
		return -1;
	}

	return result;
}

// Sends a message through a message queue
int send_message(int qid, struct messageBuffer * qbuf)
{
	int result, length;
	length = sizeof(struct messageBuffer) - sizeof(long);
	
	if (( result = msgsnd(qid,qbuf, length,0)) == -1)
	{
		return -1;
	}
	return (result);
}

// Retrieves the machine id by executing the corresponding function call in the web service components
// and sends the result to the simulation components
void getMachineID(int session_id, JNIEnv *env )
{
	int machineId = -1 ;


	jclass jni_cls = env->FindClass("SimulationService/wrapper/JavaWrapper");
	jmethodID methodId = env->GetStaticMethodID(jni_cls , "getMachineID","(I)I");
	
	machineId = env->CallStaticIntMethod(jni_class, methodId, session_id);

	messageBuffer responseMsg;
	responseMsg.mtype = CDPP_ORIGINATED;
	responseMsg.specific_type = GET_MACHINE_ID_RESPONSE;
	responseMsg.session_id = session_id;
	responseMsg.longParam1 = machineId;
	
	map<int,int>::const_iterator cursor;
	cursor = sessions_IPCSendQueues.find(session_id);
	
	if( cursor != sessions_IPCSendQueues.end() )
	{
		send_message(cursor->second,&responseMsg);
	}
}

// Retrieves the machine id of a specific model by executing the corresponding method call in the web service 
// components and sends the result to the simulation components
void machineForModel(int session_id , char* model_name,JNIEnv *env)
{
	int machineId = -1 ;	
		
        jclass jni_cls = env->FindClass("SimulationService/wrapper/JavaWrapper");
		
	jmethodID  methodId = env->GetStaticMethodID( jni_cls ,"machineForModel" , "(ILjava/lang/String;)I");

	jstring modelName = env->NewStringUTF(model_name);

	machineId = env->CallStaticIntMethod( jni_cls , methodId , session_id , modelName );

	map<int,int>::const_iterator cursor ;
	cursor=	sessions_IPCSendQueues.find(session_id);

	messageBuffer responseMsg;	
	responseMsg.mtype = CDPP_ORIGINATED;
	responseMsg.specific_type = MACHINE_FOR_MODEL_RESPONSE;
	responseMsg.session_id = session_id;
	responseMsg.longParam1 = machineId;

	if (cursor != sessions_IPCSendQueues.end())
	{
		send_message( cursor->second, &responseMsg);
	}
}	

// Delete a message queue
int remove_queue(int qid)
{
	if (msgctl( qid, IPC_RMID,0) == -1)
	{
		return -1;
	}
	return (0);

}

// Deletes the send and receive queues at the end of the simulation
void deleteQueues(int session_id)
{
	long send_queue_id = -1 ;
	long receive_queue_id  = -1 ;

	map<int,int>::iterator cursor ;

	cursor = sessions_IPCSendQueues.find(session_id);
	if ( cursor != sessions_IPCSendQueues.end() )
	{
		remove_queue(cursor->second);
	}

	sessions_IPCSendQueues.erase(cursor);

	cursor = sessions_IPCReceiveQueues.find(session_id);
	if( cursor != sessions_IPCReceiveQueues.end() )
	{
		remove_queue(cursor->second);
	}

	sessions_IPCReceiveQueues.erase(cursor);
}


// Sends a remote message to another remote machine, this is donw throw the web service components
void sendRemoteMessage(messageBuffer *msgBuffer, JNIEnv *env)
{

	jclass jni_class = env->FindClass("SimulationService/wrapper/JavaWrapper");
	jmethodID methodId = env->GetStaticMethodID(jni_class, "sendRemoteMessage", "(IILjava/lang/String;ILjava/lang/String;IDIZII)V");
	
	int session_id = msgBuffer->session_id;
	string message_time(msgBuffer->charParam1);
	int srcProcId = msgBuffer->longParam1;
	int machineId = msgBuffer->longParam4;
	int destProcId = msgBuffer->longParam5;
	string next_change = string("");
	int message_type = -1;
	bool isFromSlave= false;
	int portId = -1;
	double value ;
	int senderModelId = -1;

	message_type = msgBuffer->specific_type;
		
	switch (msgBuffer->specific_type)
	{
		case INIT_MESSAGE:
		case COLLECT_MESSAGE:
		case INTERNAL_MESSAGE:
			value = -1;
			break;
		case DONE_MESSAGE:
		{
			next_change = string(msgBuffer->charParam2);
			isFromSlave = msgBuffer->boolParam1;
			value = -1;
			break;
		}
		case EXTERNAL_MESSAGE:
		{
			portId = msgBuffer->longParam2;
			value = msgBuffer->doubleParam1;
			senderModelId = msgBuffer->longParam3;
			break;
		}
		case OUTPUT_MESSAGE:
		{
			portId = msgBuffer->longParam2;
			value = msgBuffer->doubleParam1;
			break;
		}
		case ECHO_MESSAGE:
		{
			portId = msgBuffer->longParam2;
			senderModelId = msgBuffer->longParam3;
			value  = -1;
		}
	}	
	

	jstring msgTimeStr = env->NewStringUTF( message_time.c_str() );
	jstring nextChgStr = env->NewStringUTF( next_change.c_str() );
	
	env->CallStaticVoidMethod(jni_class, methodId, session_id, message_type, msgTimeStr, srcProcId, 
		nextChgStr, portId, value, senderModelId, isFromSlave, machineId, destProcId);	        
}


// Receives a remote message from another machine and delivers it to the simulation components
JNIEXPORT void JNICALL Java_SimulationService_wrapper_JavaWrapper_receiveRemoteMessageByProxy
  (JNIEnv * env, jclass cls, jint session_id, jint message_type, jstring msg_time, jint srcProcId,
  	 jstring next_change, jint portId, jdouble value, jint senderModelId, jboolean isFromSlave
  	 , jint destProcId)
{
	
	const char* msgTime;
	const char* nextChg;
	
	messageBuffer msgBuffer;
	
	msgBuffer.session_id = session_id;
	msgBuffer.mtype = WRAPPER_ORIGINATED;
	msgBuffer.specific_type = message_type;
	
	msgTime = env->GetStringUTFChars(msg_time,0);
	strcpy(msgBuffer.charParam1, msgTime);
	env->ReleaseStringUTFChars(msg_time, msgTime);
	
	msgBuffer.longParam1 = srcProcId;
	msgBuffer.longParam5 = destProcId;

	switch( message_type )
	{
		case INIT_MESSAGE:	
		case COLLECT_MESSAGE:
		case INTERNAL_MESSAGE:
		{
			break;
		}
		case DONE_MESSAGE:
		{
			nextChg = env->GetStringUTFChars(next_change,0);
			strcpy(msgBuffer.charParam2, nextChg);
			env->ReleaseStringUTFChars(next_change, nextChg);
			msgBuffer.boolParam1 = isFromSlave;
			break;
		}
		case EXTERNAL_MESSAGE:
		{
			msgBuffer.longParam2 = portId;
			msgBuffer.doubleParam1 = value;
			msgBuffer.longParam3 = senderModelId;
			break;
		}
		case OUTPUT_MESSAGE:
		{
			msgBuffer.longParam2 = portId;
			msgBuffer.doubleParam1 = value;
			break;	
		}
		case ECHO_MESSAGE:
		{	
			msgBuffer.longParam2 = portId;
			msgBuffer.longParam3 = senderModelId;
		}
	}

	
	map<int, int >::const_iterator cursor;
	cursor = sessions_IPCSendQueues.find(session_id);

	struct timeval msgStartTime ;
	gettimeofday( &msgStartTime, (struct timezone*) 0) ;

	msgBuffer.longParam6 = msgStartTime.tv_sec;
	msgBuffer.longParam7 = msgStartTime.tv_usec;

	if( cursor != sessions_IPCSendQueues.end())
	{
		send_message( cursor->second, &msgBuffer );	
	}

}

// Sends a stop simulation message to the simulation components, used in the case of distributed simulation
JNIEXPORT void JNICALL Java_SimulationService_wrapper_JavaWrapper_stopSimulationSession (JNIEnv *env, jclass cls , jint session_id)
{
	messageBuffer msgBuffer;

	string nullStr("");
	msgBuffer.mtype = WRAPPER_ORIGINATED;
	msgBuffer.specific_type = SIMULATION_STOP;
	msgBuffer.session_id  = session_id;

	msgBuffer.longParam1 = -1;
	msgBuffer.longParam2 = -1;
	msgBuffer.longParam3 = -1;
	msgBuffer.longParam4 = -1;
	msgBuffer.longParam5 = -1;
	msgBuffer.doubleParam1 = -1;
	msgBuffer.boolParam1 = false;
	strcpy(msgBuffer.charParam1 , nullStr.c_str() );
	strcpy(msgBuffer.charParam2 , nullStr.c_str() );
	strcpy(msgBuffer.charParam3 , nullStr.c_str() );

	map<int,int>::const_iterator cursor;

	cursor = sessions_IPCSendQueues.find(session_id);

	if (cursor != sessions_IPCSendQueues.end() )
		send_message(cursor->second, &msgBuffer);
}

// Adds zone partition (Cell-DEVS models) by sending the partitions to simulation components
JNIEXPORT void JNICALL Java_SimulationService_wrapper_JavaWrapper_addZonePartition  (JNIEnv *env, jclass cls, jint session_id, jstring zone, jint machineId)
{

	messageBuffer msgBuffer ;
	const char* buff; 
	string nullStr("");

	msgBuffer.mtype = WRAPPER_ORIGINATED;
	msgBuffer.specific_type = ADD_ZONE_PARTITION;
	msgBuffer.session_id = session_id;

	msgBuffer.longParam1 = machineId;
	buff = env->GetStringUTFChars( zone, 0 );
	strcpy(msgBuffer.charParam1, buff) ;
	env->ReleaseStringUTFChars(zone, buff );	

	msgBuffer.longParam2 = -1;
	msgBuffer.longParam3 = -1;
	msgBuffer.longParam4 = -1;
	msgBuffer.longParam5 = -1;
	msgBuffer.doubleParam1 = -1;
	msgBuffer.boolParam1 = false;
	strcpy(msgBuffer.charParam2, nullStr.c_str() );
	strcpy(msgBuffer.charParam3, nullStr.c_str() );
	

	map<int,int>::const_iterator cursor;
	cursor = sessions_IPCSendQueues.find(session_id);

	if( cursor != sessions_IPCSendQueues.end() )
	{
		time_t rawtime;
		time(&rawtime);
		send_message( cursor->second, &msgBuffer );
	}
}

