/**
 *           Advanced Real-Time Simulation Lab (ARS) - Carleton University
 *
 * PROJECT: 			Web Service-Enabled DCD++ Client
 * AUTHOR(S): 			Rami Madhoun
 * PURPOSE: 			Euaclyptus
 * CMD LINE ARG:
 * OUTPUT:
 * DATE CREATED: 		Nov 25, 2006
 * LAST MOD:   			Jan 10, 2007
 * LAST MOD BY:			Rami Madhoun
 * COPYRIGHT:			ARS-Carleton University
 * USAGE:
 * COMMENTS:
 **/

package ca.gc.nrc.iit.pds.appgui.simulation;

import java.awt.BorderLayout;
import java.awt.event.*;
import java.awt.*;

import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import com.borland.jbcl.layout.XYLayout;
import com.borland.jbcl.layout.*;
import javax.swing.BorderFactory;

import javax.activation.DataHandler;
import javax.activation.FileDataSource;

import java.io.*;
import SimulationService.stubs.CDppPortType;

import SimulationService.pstubs.PCDppPortType;

import org.jdesktop.swingworker.*;

public class SaveMonitor extends JFrame {

// operation mode : DCD++ / PCD++
    private final int MODE_DISTRIBUTED = 1;
    private final int MODE_PARALLEL = 2;
    private int mode = MODE_DISTRIBUTED;

    public static String fileName = null;
    public static int sessionID ;

// DCD++/PCD++ stubs classes
    public static CDppPortType cdppPortType = null;
    public static PCDppPortType pcdppPortType = null;

    private static SaveMonitor saveMonitorInstance = null;

    XYLayout xYLayout1 = new XYLayout();
    JPanel jPanel1 = new JPanel();
    XYLayout xYLayout2 = new XYLayout();
    XYLayout xYLayout3 = new XYLayout();
    XYLayout xYLayout4 = new XYLayout();
    static JProgressBar jProgressBarMonitor = new JProgressBar();


// Swing worker class used to retrieve the log file in the background, the log file can be very big and has
// to be saved in the background to that it doesn't freeze the gui
    SwingWorker fileSaver = new SwingWorker<Void,Void>() {

    @Override
    public Void doInBackground() {

        savingStarted();

        DataHandler dhReceivedLogFile = null;
        try {

            switch(mode) {
            case MODE_DISTRIBUTED:
                dhReceivedLogFile = SaveMonitor.cdppPortType.retrieveLogFile(SaveMonitor.sessionID);
                break;
            case MODE_PARALLEL:
                dhReceivedLogFile = SaveMonitor.pcdppPortType.retrieveLogFiles(SaveMonitor.sessionID);
                break;
            }

            File receivedLogFile = new File(dhReceivedLogFile.getName());

            if (receivedLogFile.exists()) {
                File savedLogFile = new File(SaveMonitor.fileName);
                BufferedReader in = new BufferedReader(new FileReader(receivedLogFile));
                BufferedWriter out = new BufferedWriter(new FileWriter(savedLogFile));

                int c = -1;

                while( ( c= in.read() ) != -1) {
                    out.write((char) c);
                }
                in.close();
                out.close();
            }

            dhReceivedLogFile = null;

            savingStopped();

        }catch (Exception excep) {
            excep.printStackTrace();
        }
     return null;
    }
};

// In the case of using DCD++ ( mode = MODE_DISTRIBUTED )
    public SaveMonitor(int sessionID, String fileName, CDppPortType cdppPortType) {


// Sets the object reference, sessionID, log file name used to save the file locally, and the stub
// class instance

            saveMonitorInstance = this;
            this.sessionID = sessionID;
            this.fileName = fileName;
            this.cdppPortType = cdppPortType;

            mode = MODE_DISTRIBUTED;
            jbInit();

            fileSaver.execute();
     }

// In the case of PCD++ (MODE_PARALLEL)
     public SaveMonitor(int sessionID , String fileName, PCDppPortType pcdppPortType) {

// Sets the object reference, sessionID, log file name used to save the file locally, and the stub
// class instance

         saveMonitorInstance = this;
         this.sessionID = sessionID;
         this.fileName = fileName;
         this.pcdppPortType = pcdppPortType;

         mode = MODE_PARALLEL;

         jbInit();

         fileSaver.execute();
     }

// Updates the progress bar during the save process
    public static void savingStarted() {
        jProgressBarMonitor.setEnabled(true);
        jProgressBarMonitor.setStringPainted(true);
        jProgressBarMonitor.setString("Saving the  log file!");
        jProgressBarMonitor.setIndeterminate(true);
    }

// Updates the progress bar after finishing the save process
    public static void savingStopped () {
        jProgressBarMonitor.setEnabled(false);
        jProgressBarMonitor.setStringPainted(false);
        jProgressBarMonitor.setString("The log file was saved!");
        jProgressBarMonitor.setIndeterminate(false);

        saveMonitorInstance.dispose();
    }

    private void jbInit()  {

        try {
            this.setDefaultLookAndFeelDecorated(true);
            this.setSize(new Dimension(160,45));
            this.setLocationRelativeTo(null);
            this.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);

            jProgressBarMonitor.setIndeterminate(true);
            jProgressBarMonitor.setStringPainted(true);
            jProgressBarMonitor.setString("");

            getContentPane().setLayout(xYLayout1);
            jPanel1.setBorder(BorderFactory.createEtchedBorder());
            jPanel1.setLayout(xYLayout2);
            xYLayout1.setWidth(261);
            xYLayout1.setHeight(79);
            this.setResizable(false);
            this.getContentPane().add(jPanel1, new XYConstraints(5, 5, 251, 69));
            this.addWindowListener(new SaveMonitor_this_windowAdapter(this));
            jPanel1.add(jProgressBarMonitor, new XYConstraints(48, 20, 145, 24));
        } catch (Exception excep) {
            excep.printStackTrace();
        }
        this.getContentPane().setLayout(xYLayout4);
        jPanel1.setLayout(xYLayout3);
        xYLayout4.setWidth(276);
        xYLayout4.setHeight(81);
        this.getContentPane().add(jPanel1, new XYConstraints(2, 2, 271, 77));
        jPanel1.add(jProgressBarMonitor, new XYConstraints(36, 24, 188, 26));
    }

    public void this_windowClosing(WindowEvent e) {
        jProgressBarMonitor.setString("");
        jProgressBarMonitor.setIndeterminate(false);
    }

}

class SaveMonitor_this_windowAdapter extends WindowAdapter {
    private SaveMonitor adaptee;
    SaveMonitor_this_windowAdapter(SaveMonitor adaptee) {
        this.adaptee = adaptee;
    }

    public void windowClosing(WindowEvent e) {
        adaptee.this_windowClosing(e);
    }

}
