/*******************************************************
*
*  DESCRIPTION: Exceptions classes 
*
*  AUTHOR: Mariana C. D'Abreu
*
*  EMAIL: mdabreu@dc.uba.ar
*
********************************************************/


#ifndef EXCEPTS_H
#define EXCEPTS_H

#include "except.h"


//******************************************************
//
// Class ECircuitException
//
// Description: excepcion arrojada por los circuitos 
//		elctricos
//
//******************************************************

class ECircuitException : public MException {

  public:

    ECircuitException( const string &str = "ECircuit Exception" ): MException( str )
      {};

    virtual ~ECircuitException() {}; 
};


//******************************************************
//
// Class ECompException
//
// Description: excepcion arrojada por los componentes  
//		de un circuito elctrico
//
//******************************************************

class ECompException : public MException {

  public:

    ECompException( const string &str = "EComp Exception" ): MException( str )
      {};

    virtual ~ECompException() {};
};


//******************************************************
//
// Class PortCompException
//
// Description: excepcion arrojada por los componentes  
//		con pines de un circuito elctrico
//
//******************************************************

class PortCompException : public ECompException {

  public:

    PortCompException( const string &str = "PortComp Exception" ): ECompException( str )
      {};

    virtual ~PortCompException() {}; 
};


//******************************************************
//
// Class OnePortException
//
// Description: excepcion arrojada por los componentes  
//		con dos pines de un circuito elctrico
//
//******************************************************

class OnePortException : public PortCompException {

  public:

    OnePortException( const string &str = "OnePort Exception" ): PortCompException( str )
      {};

    virtual ~OnePortException() {}; 
};


//******************************************************
//
// Class TwoPortException
//
// Description: excepcion arrojada por los componentes  
//		con cuatro pines de un circuito elctrico
//
//******************************************************

class TwoPortException : public PortCompException {

  public:

    TwoPortException( const string &str = "TwoPort Exception" ): PortCompException( str )
      {};

    virtual ~TwoPortException() {}; 
};


//******************************************************
//
// Class BondGraphException
//
// Description: excepcion arrojada por los grafos Bond Graph
//
//******************************************************

class BondGraphException : public MException {

  public:

    BondGraphException( const string &str = "BondGraph Exception" ): MException( str )
      {};

    virtual ~BondGraphException() {};
};


//******************************************************
//
// Class BGCompException
//
// Description: excepcion arrojada por los componentes de
//		los grafos Bond Graph 
//
//******************************************************

class BGCompException : public MException {

  public:

    BGCompException( const string &str = "BGComp Exception" ): MException( str )
      {};

    virtual ~BGCompException() {};
};


//******************************************************
//
// Class EC_BG_MapperException
// 
// Description: excepcion arrojada por el mapeador de 
//		circuitos elctricos a Bond Graph 
//
//******************************************************

class EC_BG_MapperException : public MException {

  public:

    EC_BG_MapperException( const string &str = "EC to BG Mapper Exception" ): MException( str )
      {};

    virtual ~EC_BG_MapperException() {}; 
};


//******************************************************
//
// Class BG_CDPP_CompilerException
//
// Description: excepcion arrojada por el compilador
//		de grafos Bond Graph a modelos DEVS
//
//******************************************************

class BG_CDPP_CompilerException : public MException {

  public:

    BG_CDPP_CompilerException( const string &str = "BG to CD++ Compiler Exception" ): MException( str )
      {};

    virtual ~BG_CDPP_CompilerException() {}; 
};


#endif

