/*******************************************************
*
*  DESCRIPTION: QBG Gyrator class (effort input) 
*
*  AUTHOR: Mariana C. D'Abreu
*
*  EMAIL: mdabreu@dc.uba.ar
*
********************************************************/

#ifndef _QBG_GYRATOR_EI_H
#define _QBG_GYRATOR_EI_H

#include "atomic.h"  
#include "fstream.h"

#define QBG_GYRATOR_EI_CLASS_NAME	 "QBGGyratorEffortIn"

class QBGGyratorEffortIn : public Atomic {

  public:
  
    QBGGyratorEffortIn( const string &name = QBG_GYRATOR_EI_CLASS_NAME );    
    ~QBGGyratorEffortIn();
  
    virtual string className() const { return QBG_GYRATOR_EI_CLASS_NAME;}
  
    static const RealValue DefaultGyrationConductance;

  protected:
  
    Model     &initFunction() ;
    Model     &externalFunction( const ExternalMessage & );
    Model     &internalFunction( const InternalMessage & );
    Model     &outputFunction( const InternalMessage & );
  
    QBGGyratorEffortIn &calculateValues();
    
  private:

    const Port	&e1p, &e2p;
    Port	&f1p, &f2p;

    RealValue 	f1, f2;  	// flows 
    RealValue	e1, e2;    	// efforts
    RealValue   G;        	// constante del gyrator 

    ofstream    outFile;        // out file
    string      outFileName;   
};  

#endif                                                                        
