/*******************************************************
*
*  DESCRIPTION: QBG Gyrator class (flow input)
*
*  AUTHOR: Mariana C. D'Abreu
*
*  EMAIL: mdabreu@dc.uba.ar
*
********************************************************/

#ifndef _QBG_GYRATOR_FI_H
#define _QBG_GYRATOR_FI_H

#include "atomic.h"  
#include "fstream.h"

#define QBG_GYRATOR_FI_CLASS_NAME	 "QBGGyratorFlowIn"

class QBGGyratorFlowIn : public Atomic {

  public:
  
    QBGGyratorFlowIn( const string &name = QBG_GYRATOR_FI_CLASS_NAME );    
    ~QBGGyratorFlowIn();
  
    virtual string className() const { return QBG_GYRATOR_FI_CLASS_NAME;}
  
    static const RealValue DefaultGyrationConductance;

  protected:
  
    Model     &initFunction() ;
    Model     &externalFunction( const ExternalMessage & );
    Model     &internalFunction( const InternalMessage & );
    Model     &outputFunction( const InternalMessage & );
  
    QBGGyratorFlowIn &calculateValues();
    
  private:

    const Port	&f1p, &f2p;
    Port	&e1p, &e2p;

    RealValue 	f1, f2;  	// flows 
    RealValue	e1, e2;    	// efforts
    RealValue   G;        	// constante del gyrator 

    ofstream    outFile;        // out file
    string      outFileName;  
};  

#endif                                                                        
