/*******************************************************
*
*  DESCRIPTION: QBG transformer class 
*
*  AUTHOR: Mariana C. D'Abreu
*
*  EMAIL: mdabreu@dc.uba.ar
*
********************************************************/

#ifndef _QBG_TRANSFORMER_H
#define _QBG_TRANSFORMER_H

#include "atomic.h"  
#include "fstream.h"

#define QBG_TRANSFORMER_CLASS_NAME	 "QBGTransformer"

class QBGTransformer : public Atomic {

  public:
  
    QBGTransformer( const string &name = QBG_TRANSFORMER_CLASS_NAME );    
    ~QBGTransformer();
  
    virtual string className() const { return QBG_TRANSFORMER_CLASS_NAME;}
  
    static const RealValue DefaultTurnsRatio;

  protected:
  
    Model     &initFunction() ;
    Model     &externalFunction( const ExternalMessage & );
    Model     &internalFunction( const InternalMessage & );
    Model     &outputFunction( const InternalMessage & );
  
    QBGTransformer &calculateValues();
    
  private:

    const Port	&e1p, &f2p;
    Port	&e2p, &f1p;

    RealValue 	f1, f2;  	// flows 
    RealValue	e1, e2;    	// efforts
    RealValue   T;        	// constante de transformacion 

    ofstream    outFile;    
    string	outFileName;
};  

#endif                                                                        
