/*  
 * v1.0 Edited by Jing and Omair   (05/11/2005)
 * -Fixed FileDialog box; made modal with respect to "this class" so that when it is clicked it stays on top.
 * 
 * -Fixed Save as .bat --> so that now when there's incorrect *.ma filename or if there's no filename
 *  you won't be able to save a *.bat file. 
 * 
 * -Fixed Error in testString()
 * currently the function is setup so that it seeks numbers in the string [no matter where] and contatenates them, adds 1 to the concatenated number and appends the number to the end of the string. 
 * so if you make a file name called 4567i and you overwrite the file [using automatic] you will get a new file called: 4567i4568 when it should be 4567i1
 * Fix: rewrote the function by checking from the back instead of from the front to find the location of a character. Then the numbers [after the char] will be chopped off as a substring and returned.
 * 
 * -Fixed Error in setDigit()
 * When trying to save a *.bat file called "2" and saving another file called "2" using the automatic save will result in either window not responding or nothing happens. 
 * Fix: altered the setDigit() [no more index out of bounds exception] so that when filename is a number it returns a string representation of the newly updated number.
 * 
 * -Fixed Error when closing window
 * Bug: When window is first opened, it will not close when clicking on the "X" (close button at the top right corner of window)
 * Fix: set OK2Close = true by default so that window will close.
 *
 * -Fixed Error for when canceling browse
 * BUG: When there's a file in the text box [text box filled] beside the browse button and if browse is clicked and you hit cancel to "cancel" file selection. 
 * The file name will be erased from the text box [empty text box] and you will have to reselect the file. 
 * Fix: Check for if the file returned from the file selection is equal to "" [nothing] if so then don't update text box field.
 * 
 * -Fixed error which occurs if the root directory of eclipse has spaces in it
 * Bug: If the root directory of eclipse has spaces in it then this button didn't function at all
 * Fix: We need to enclose IPath objects namely path and internalPath in double quotes by simply concatenating
 * their string representations by "/""
 * 
 * -Fixed error which occurs if any fileNames have a space in them
 * Bug: If the for e.g. say a .val file has spaces in it then an error will occur terminating the function 
 * immediately
 * Fix: All filenames were enclosed in double quotes which was done in collect parameters

 * 
 */
 
package CELLDEV.buttons;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.PlatformUI;

import CDBuilder.CDBuilderPlugin;
import CDBuilder.buttons.OverwriteGui;
import CDBuilder.console.ConsoleDocument;
import CDBuilder.console.StreamHandler;


/**
 * @author Ali Monadi
 *
 * This class is responsible for the popup gui which appears when the Bmp to value button  
 * is pressed
 */

public class BmpTranslatorGUI extends Dialog{

	private Shell parent;
	
	private String title;
	private String message1;
	private String message2;
	private String message3;
	private String message4;
	private String message5;
	private String precision;
	private String bmpFile;
	private String valFile;
	private String coupleMessage;
	
	private Button simuButton;
	private Button doneButton;
	private Button killButton;
	
	private Button saveSetButton;
	private Button loadSetButton;
	private Button saveBatButton;
	private Button loadBatButton;
	private Button checkBoxAdv;
	private Button checkBoxPrecision;
	private Button checkBoxComment;
	private Combo coupleCombo;

	private Text textMa;
	private Text textBmp;
	private Text textMax,textMin;
	private Text textPrecision;
	private Text textVal;
	private Text textAdv;
	private Text textComment;

	private static boolean checkedPrecision = false; 
	private static boolean checkedAdv = false;
	private static boolean checkedComment = false;

	private boolean simuKilled = false;
	private RunSimu rsimu;
	
	private Process simuProcess;
	private String maFileName;
	private IPath path;
	private IPath internalPath;
	
	//window will only cose if OK2Close is true. This is set to true automatically at beginning [not set in constructor]
	private boolean OK2Close=true;
	private Composite composite;
	private int n;
	
	class conWrite implements Runnable {
		String line;
		public conWrite(String line) {
			this.line = line;
		}
		public void run() {
			ConsoleDocument.getCDOS().set(line);
		}
	}

	class RunSimu extends Thread {
		Dialog d;
		String command;

		
		public RunSimu(String command, Dialog d) {
			this.command = command;
			this.d = d;
		}
		
		public void run() {
			try {
				
				ConsoleDocument cdos = ConsoleDocument.getCDOS();
				IContainer container = ResourcesPlugin.getWorkspace().getRoot().getContainerForLocation(path);				
							
				FileWriter tempFile = new FileWriter(path.append("tempSimu.bat").toString());
				
				//testing purposes
				tempFile.write("\""+internalPath+"bmptoval\"" + command);  //internalPath is where the cd++.exe is located
				tempFile.close();
				
				simuProcess = Runtime.getRuntime().exec("cmd /c \"" + path.toString() + "tempSimu.bat\"", null, new File(path.toString()));
				StreamHandler outputHandler = new StreamHandler(simuProcess.getInputStream(), "OUTPUT", true);
				StreamHandler errorHandler = new StreamHandler(simuProcess.getErrorStream(), "ERROR", true);            
			
				outputHandler.start();
				errorHandler.start();

				int exitVal = simuProcess.waitFor();

				container.refreshLocal(1,null);
				ResourcesPlugin.getWorkspace().getRoot().getFileForLocation(path.append("tempSimu.bat")).delete(true,null);

				Display.getDefault().syncExec(new Runnable() {
					public void run(){	
						doneButton.setEnabled(true);
						killButton.setEnabled(false);
						simuButton.setEnabled(true);
						d.close();
					}
				});
				
				if (simuKilled) {
							
					Display.getDefault().syncExec(new conWrite("***User has killed the simulation***" + "\n"));
					simuKilled = false;
				
				}

			}
			catch (Throwable T) {
				T.printStackTrace();
			}
		}
	}// end of class RunSimu



	/*
	 * constructor
	 */
	public BmpTranslatorGUI(Shell parentShell,IPath path, IPath internalPath, String maFileName) {
		super(parentShell);
		parent = parentShell;
		//this.title = "Image Data to Value";
		this.title = "Convert Bmp to VAL";
		message1 = "Model file (.ma)";
		message3 = "Normalization Bounds:";
		message4 = "Advanced Users Only. Enter desired parameters:";
		message5 = "Comments";
		precision = "Enter Precision:";
		bmpFile =  "Bmp file (.bmp)";
		valFile =  "Val file (.val)";
		
		coupleMessage = "Cellular model name";
		this.maFileName = maFileName;
		this.internalPath = internalPath;
		this.path = path;
	}
	
	
	/*
	 * Determines if a text box is left empty and displays error message
	 * @return false of empty true otherwise
	 */
	
	private boolean isEmpty(String parameter,Text textBox){
		String value = textBox.getText().trim();	
		if(value.length()== 0){
				
			MessageBox mbox = new MessageBox(new Shell(),SWT.ICON_ERROR | SWT.OK);
			mbox.setText("Simulation Stopped");
			mbox.setMessage(parameter+" is not entered");
			mbox.open();
			
			
			doneButton.setEnabled(true);
			killButton.setEnabled(false);
			simuButton.setEnabled(true);
			return true;
		}
		return false;
	}
	
	
	/*
	 * checks to see if the given string matches a file name found in the project directory
	 * 
	 * @param String file, name of file 
	 * @param String errMsg, message to be displayed IF file was not found
	 */	

	private boolean checkFileExistence(String file, String errMsg){
	
		IWorkspaceRoot root = (IWorkspaceRoot)PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().getInput();
		IResource[] files;
		IFile fileHandle = null;

		try {
			files = root.getContainerForLocation(path).members();

			for(int i=0;i<files.length;i++){
			
				if(files[i]instanceof IFile && files[i].getName().equalsIgnoreCase(file)){
					fileHandle = (IFile) files[i];
					break;
				}
			}
			if (fileHandle == null){

				MessageBox mbox = new MessageBox(new Shell(), SWT.ICON_ERROR | SWT.OK);
				mbox.setText("Simulation Stopped");
				mbox.setMessage(errMsg);
				mbox.open();
							
				return false;
			}
			return true;
			
		} catch (CoreException e) {
			MessageBox mbox = new MessageBox(new Shell(), SWT.ICON_ERROR | SWT.OK);
			mbox.setText("Simulation Stopped");
			mbox.setMessage("Error occured while accessing file");
			mbox.open();
					
			e.printStackTrace();
			return false;	
		}			
	}
	
	
	/*
	 * This function will check to see if the file name entered in the 
	 * 
	 * @param Text name, the text box value containing the file name
	 * @param String extention, the extention the name shoule have
	 * @return a string containing the file name with the proper extention
	 */
	
	public String checkExtention(Text name , String extention){
		String fileName = name.getText().trim();
		
		if(!fileName.endsWith(extention) && !fileName.endsWith(extention.toUpperCase()))
			return fileName.concat(extention); 
		return fileName;
	}
	
	
	/*
	 * This function will record all the parameters entered into a string
	 *
	 * @return String will contain all parameters. if there is an error found NULL will be returned
	 */
	public String collectParameters(){
		String[] param = new String[9];

			if(isEmpty("Model file",textMa) || isEmpty("Bitmap file",textBmp) || isEmpty("Min value",textMin) 
			|| isEmpty("Value file",textVal) || isEmpty("Max value",textMax)){
				return null;
			}
			
			if( !checkFileExistence(checkExtention(textMa,".ma"), "Ma file does not exist") 
			 || !checkFileExistence(checkExtention(textBmp,".bmp"), "Bmp file does not exist")){
				return null;
			}	
			
			param[0] = "-m\"" + checkExtention(textMa,".ma")+"\"";
			param[1] = "-c" + coupleCombo.getText().trim();
			param[2] = "-b\"" + checkExtention(textBmp,".bmp")+"\"";
			param[3] = "-l" + textMin.getText().trim();
			param[4] = "-u" + textMax.getText().trim();
			param[5] = "-v\"" + checkExtention(textVal,".val")+"\"";
			//percision is an optional parameter
			if(textPrecision.isEnabled()) {
				if(isEmpty("Precision value", textPrecision))
					return null;
				param[6] = "-p" + textPrecision.getText().trim(); 
			}	
		
			if(textAdv.isEnabled()){

				if(isEmpty("advanced value", textAdv))
					return null;
				param[7] = textAdv.getText().trim();
			}
	
			String combine = "";
			for (int n = 0;  n < 8; n++ ) {
				if (param[n] != null)
					combine = combine + " " + param[n];
			}
		combine = combine.concat( " " );
		return combine;
	}
	
	
	
	/*
	 * Performs proper action based on the botton pressed(Close or Proceed)
	 * 
	 * @param int buttonId
	 */
	protected void buttonPressed(int buttonId) {


		if (buttonId == IDialogConstants.CLOSE_ID){

			OK2Close = true;
			close();
		}
		if (buttonId == IDialogConstants.PROCEED_ID) {

			
			String command = collectParameters();
			if (command == null)
				return;
			
			System.out.println(command);
			rsimu = new RunSimu(command,this);
			rsimu.start();
		} 
		super.buttonPressed(buttonId);
	}



	protected void configureShell(Shell shell) {
		super.configureShell(shell);
		//the following line registers the shell of SimuGui to parent
		//which causes the file-dialog box modal to the SimuGui dialog
		//this fixes the problem associated with the file-dialog not being modal to SimuGui 
		parent = shell;
		if (title != null)
			shell.setText(title);
	}




	protected void createButtonsForButtonBar(Composite parent) {
		// create OK and Cancel buttons by default
		simuButton = createButton(parent, IDialogConstants.PROCEED_ID, IDialogConstants.PROCEED_LABEL, true);
		simuButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e){
				killButton.setEnabled(true);
				simuButton.setEnabled(false);
				doneButton.setEnabled(false);
			}
		});
		doneButton = createButton(parent, IDialogConstants.CLOSE_ID, IDialogConstants.CLOSE_LABEL, true);
		killButton = createButton(parent, IDialogConstants.STOP_ID, "Kill", true);
		killButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e){
				simuProcess.destroy();
				try {
					Process lookProcess = Runtime.getRuntime().exec("ps");
					BufferedReader bf = new BufferedReader(new InputStreamReader(lookProcess.getInputStream()));
					String msg = null;
					while((msg = bf.readLine())!= null){
						if (msg.indexOf(path.toString().substring(2))!= -1){
							msg = msg.substring(2).trim();
							Process killProcess = Runtime.getRuntime().exec("kill -9 " + msg.substring(0,msg.indexOf(" ")));
						}
					}
				} catch (IOException e1) {
					e1.printStackTrace();
				}
				doneButton.setEnabled(true);
				killButton.setEnabled(false);
				simuButton.setEnabled(true);
				simuKilled = true;
			}
		});
		
		killButton.setEnabled(false);
		//do this here because setting the text will set enablement on the ok button
		textMa.setFocus();
	
		
	}
	
	
	
	/*
	 * This is the layout of the GUI which will appear when the button is pressed in the 
	 * tool bar
	 * 
	 * @param Composite parent
	 */	
	
	protected Control createDialogArea(Composite parent) {
		// create composite( which is a control that can contain other controls!)
		composite = (Composite)super.createDialogArea(parent);
		GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 10;
		composite.setLayout(gridLayout);

        //Row One - Should be text description of ma text box
		Label label = createLabel(parent, composite, message1);

		// Row two - Text Box for the Ma file and file button
		
		/*this section will look throught the project directory and check if the model file exist
		* If it does exists places the names of the cellular models found in the file into the 
		* cellular model text box (coupleCombo)
		*/
		textMa = new Text(composite, SWT.SINGLE | SWT.BORDER);
		if (maFileName != null)
			textMa.setText(maFileName);
		textMa.addModifyListener( new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				coupleCombo.removeAll();
				boolean modelExist = false;
				IWorkspaceRoot root = (IWorkspaceRoot)PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().getInput();
				
				try {
					IResource[] files;
					IFile dropFile = null;
					files = root.getContainerForLocation(path).members();
					for(int i=0;i<files.length;i++){
						if(files[i]instanceof IFile && files[i].getName().equalsIgnoreCase( checkExtention(textMa,".ma") )){
							dropFile=(IFile) files[i];
							break;
						}
					}
					if (dropFile == null){
						modelExist=false;
					}else{
						modelExist=true;
					}
				
					if(modelExist){
										
						InputStream is = dropFile.getContents();
						BufferedReader br 	= new BufferedReader(new InputStreamReader(is));
						String line,couple = null;
						while ((line=br.readLine()) != null) {
	
							if(line.startsWith("["))
								couple=line.substring(1,line.lastIndexOf("]"));
							if (line.replaceAll(" ","").equalsIgnoreCase("type:cell"))
								coupleCombo.add(couple);		
						}
						if (coupleCombo.getItemCount()>0)
							coupleCombo.setText(coupleCombo.getItem(0));
					}
				} catch (Exception e1) {
					e1.printStackTrace();
				}	
			}
		});//end of listener
		
		Color c = new Color(null,255,255,255);
		textMa.setBackground(c);
		GridData gridData = new GridData();
		gridData.horizontalSpan=9;
		gridData.widthHint = convertHorizontalDLUsToPixels(IDialogConstants.MINIMUM_MESSAGE_AREA_WIDTH);
		textMa.setLayoutData(gridData);
		Button fileButtonMa = new Button(composite, SWT.PUSH);
		fileButtonMa.setText("Browse");
		fileButtonMa.setToolTipText("Select Ma File");
		GridData gridDataFBMa = new GridData();
		gridDataFBMa.horizontalAlignment = GridData.BEGINNING;
		fileButtonMa.setLayoutData(gridDataFBMa);
		fileButtonMa.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e){
			composite.setEnabled(false);
			simuButton.setEnabled(false);
			doneButton.setEnabled(false);
			saveBatButton.setEnabled(false);
			loadBatButton.setEnabled(false);
			OK2Close = false;
			String fileName = NewFileGui("*.Ma");
			if (!fileName.equals(""))
			textMa.setText( fileName);
			}
		});
		

//		Row Three - Should be text description of ma text box
		label = createLabel(parent, composite, bmpFile);
		
//		Row Four - Text Box for the bmp file name
		 textBmp = new Text(composite, SWT.SINGLE | SWT.BORDER);

		 textBmp.setBackground( c );
		 gridData = new GridData();
		 gridData.horizontalSpan = 9;
		 gridData.grabExcessHorizontalSpace = true;
		 gridData.widthHint = convertHorizontalDLUsToPixels(IDialogConstants.MINIMUM_MESSAGE_AREA_WIDTH);
		 textBmp.setLayoutData( gridData );
		 Button fileButtonBmp = new Button( composite, SWT.PUSH );
		 fileButtonBmp.setText( "Browse" );
		 fileButtonBmp.setToolTipText( "Select Bmp File" );
		 
		 GridData gridDataFBBmp = new GridData();
		 gridDataFBBmp.horizontalAlignment = GridData.BEGINNING;
		 fileButtonBmp.setLayoutData( gridDataFBBmp );
		 fileButtonBmp.addSelectionListener( new SelectionAdapter() {
			 public void widgetSelected( SelectionEvent e ){
			 composite.setEnabled(false);
			 simuButton.setEnabled(false);
			 doneButton.setEnabled(false);
			 saveBatButton.setEnabled(false);
			 loadBatButton.setEnabled(false);
			 OK2Close = false;
			 String fileName = NewFileGui( "*.bmp" );
			 if (!fileName.equals(""))
			 textBmp.setText( fileName );
			 }
		 });		

		//Row Five - Should be text description of ma text box
		label = createLabel(parent, composite, valFile);

		//Row Six - Text Box for the second VA file
		textVal = new Text(composite, SWT.SINGLE | SWT.BORDER);
		textVal.setBackground( c );
		gridData = new GridData();
		gridData.horizontalSpan = 9;
		gridData.widthHint = convertHorizontalDLUsToPixels(IDialogConstants.MINIMUM_MESSAGE_AREA_WIDTH);
		textVal.setLayoutData( gridData );

		Button fileButtonVal = new Button( composite, SWT.PUSH );
		fileButtonVal.setText( "Browse" );
		fileButtonVal.setToolTipText( "Select Val File" );
		 
		GridData gridDataFBVal = new GridData();
		gridDataFBVal.horizontalAlignment = GridData.BEGINNING;
		fileButtonVal.setLayoutData( gridDataFBVal );
		fileButtonVal.addSelectionListener( new SelectionAdapter() {
			public void widgetSelected( SelectionEvent e ){
			composite.setEnabled(false);
			simuButton.setEnabled(false);
			doneButton.setEnabled(false);
			saveBatButton.setEnabled(false);
			loadBatButton.setEnabled(false);
			OK2Close = false;
			String fileName = NewFileGui( "*.Val" );
			if (!fileName.equals(""))
			textVal.setText( fileName );
			}
		});

		//Row Seven - A text description of the couple label
		label = createLabel(parent, composite, coupleMessage);
		
		//Row Eight - Text Drop Box of the couple Loader
		gridData = new GridData();
		coupleCombo = new Combo (composite, SWT.READ_ONLY);
		gridData.horizontalSpan =9;
		gridData.horizontalAlignment = GridData.FILL;
		gridData.widthHint = convertHorizontalDLUsToPixels(IDialogConstants.MINIMUM_MESSAGE_AREA_WIDTH);
		coupleCombo.setLayoutData(gridData);

		//Row Nine - Text description of the normalization values (max/min)text box
		label = createLabel(parent, composite, message3);

			
			
		label = new Label(composite,SWT.NONE);
		label.setText("Min:");
		textMin = new Text(composite, SWT.SINGLE | SWT.BORDER);
		textMin.setTextLimit(14);
		gridData = new GridData();
		gridData.widthHint = 60;
		gridData.heightHint = 13;
		textMin.setLayoutData (gridData);
		textMin.setEnabled(true);
		textMin.setToolTipText("Min value:");
		textMin.setText("0");
		
		textMin.addKeyListener(new KeyListener(){
			public void keyPressed(KeyEvent e) {
				if (!textMin.getText().trim().equals("-")){// this is so the user can enter (-)ve numbers
					try{
						Integer value= new Integer(textMin.getText());
						if(e.keyCode == SWT.ARROW_UP){
							value = new Integer(value.intValue()+1);
							textMin.setText(value.toString());
					
						}else if(e.keyCode==SWT.ARROW_DOWN){
							value = new Integer(value.intValue()-1);
							textMin.setText(value.toString());
						}

					}catch(NumberFormatException e2){
						textMin.setText("");     //clear textbox if error
					}catch(Exception e1){
						e1.printStackTrace();
					}
				}
			}
				public void keyReleased(KeyEvent e) {
				
			}
		});	
		
//		Row ten -  and text box for time
		  label = new Label(composite,SWT.NONE);
		  label.setText("Max:");
		  gridData = new GridData();
		  gridData.horizontalAlignment = GridData.BEGINNING;
		
		  textMax = new Text(composite, SWT.SINGLE | SWT.BORDER);
		  textMax.setTextLimit(14);
		
		  gridData = new GridData();
		  gridData.widthHint = 60;
		  gridData.heightHint = 13;
		  textMax.setLayoutData (gridData);
		  textMax.setEnabled(true);
		  textMax.setToolTipText("Max value:");
		  textMax.setText("0");
		  textMax.addKeyListener(new KeyListener(){
			  public void keyPressed(KeyEvent e) {
			
				  if (!textMax.getText().trim().equals("-")){// this is so the user can enter (-)ve numbers 
			
					  try{
						  Integer value= new Integer(textMax.getText());
						  if(e.keyCode == SWT.ARROW_UP){
							  value = new Integer(value.intValue()+1);
							  textMax.setText(value.toString());
						  }else if(e.keyCode == SWT.ARROW_DOWN){
							  value = new Integer(value.intValue()-1);
							  textMax.setText(value.toString());
						  }
		
					  }catch(NumberFormatException e2){
						  textMax.setText("");    //clear textbox if error
					  }catch(Exception e1){
						  e1.printStackTrace();
					  }
				  }
			  }
		
			  public void keyReleased(KeyEvent e) {
					
			  }
		 });	

		//Row Eleven - Option to set precision ( by default, bmptoval.exe will set it to "4" )					
		label = createLabel(parent, composite, precision);

		//Row Twelve - The check box and text box for advanced		
		checkBoxPrecision = new Button(composite, SWT.CHECK);
		GridData gridData1P2 = new GridData();
		gridData1P2.horizontalAlignment = GridData.BEGINNING;
		checkBoxPrecision.setLayoutData(gridData1P2);
		checkBoxPrecision.setSelection(checkedPrecision);
		checkBoxPrecision.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e){
				
				if(textPrecision.getEnabled() == true) { 
					textPrecision.setEnabled(false);
					checkedPrecision = false;
				}	
				else { 
					textPrecision.setEnabled(true);
					checkedPrecision = true;
				}
			}
		});
		
		textPrecision = new Text(composite, SWT.SINGLE | SWT.BORDER);
		textPrecision.setTextLimit(2);
		gridData = new GridData();
		gridData.widthHint = 15;
		gridData.heightHint = 13;
		textPrecision.setLayoutData (gridData);
		textPrecision.setEnabled(checkedPrecision);
		textPrecision.setToolTipText("precision");
		textPrecision.setText("4");

		textPrecision.addKeyListener(new KeyListener(){
			public void keyPressed(KeyEvent e) {
				try{// to check number format errors
					Integer max = new Integer(textPrecision.getText());

				}catch(NumberFormatException e2){
					textPrecision.setText("");    //
				}
			}
				public void keyReleased(KeyEvent e) {
		
			}
		});	



		//Row Thirteen - Text Discription of Advanced					
		label = createLabel(parent, composite, message4);

		//Row Fourteen - The check box and text box for advanced		
		checkBoxAdv = new Button(composite, SWT.CHECK);
		GridData gridData17 = new GridData();
		gridData17.horizontalAlignment = GridData.BEGINNING;
		checkBoxAdv.setLayoutData(gridData17);
		checkBoxAdv.setSelection(checkedAdv);
		checkBoxAdv.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e){
				if(textAdv.getEnabled() == true) { 
					textAdv.setEnabled(false);
					checkedAdv = false;
				}	
				else { 
					textAdv.setEnabled(true);
					checkedAdv = true;
				}
			}
		});
		
		textAdv = new Text(composite, SWT.SINGLE | SWT.BORDER);
		GridData gridData18 = new GridData();
		gridData18.horizontalAlignment = GridData.FILL;
		gridData18.grabExcessHorizontalSpace = true;
		gridData18.horizontalSpan = 9;
		textAdv.setLayoutData(gridData18);
		textAdv.setEnabled(false);

//		Row fifteen - Text description of the Comment text box
		Label label8 = createLabel(parent, composite, message5);
		
//		Row Sixteen - The check box and text box for comment
		checkBoxComment = new Button(composite, SWT.CHECK);
		GridData gridData20 = new GridData();
		gridData20.horizontalAlignment = GridData.BEGINNING;
		checkBoxComment.setLayoutData(gridData20);
		checkBoxComment.setSelection(false);
		checkBoxComment.addSelectionListener(new SelectionAdapter() {
			 public void widgetSelected(SelectionEvent e){
				  if(textComment.getEnabled() == true) { 
					  textComment.setEnabled(false);
					  checkedComment = false;	
				  }
				  else { 
					  textComment.setEnabled(true);
					  checkedComment = true;
				  }
			  }
		});			
		textComment= new Text(composite, SWT.SINGLE | SWT.BORDER);
		GridData gridData21 = new GridData();
		gridData21.horizontalSpan = 9;
		gridData21.horizontalAlignment = GridData.FILL;
		gridData21.grabExcessHorizontalSpace = true;
		textComment.setLayoutData(gridData21);
		textComment.setEnabled(false);
	  
//		Row Seventeen - Save as Batch button
	    saveBatButton = new Button(composite, SWT.PUSH);
		saveBatButton.setText("Save as .bat");
		saveBatButton.setToolTipText("Save Settings to batch file");
		GridData gridDataSBB= new GridData();
		gridDataSBB.horizontalAlignment = GridData.END;
		gridDataSBB.horizontalSpan = 9;	
		saveBatButton.setLayoutData(gridDataSBB);
		saveBatButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e){
				 composite.setEnabled(false);
				 simuButton.setEnabled(false);
				 doneButton.setEnabled(false);
				 saveBatButton.setEnabled(false);
				 loadBatButton.setEnabled(false);
				 OK2Close = false;
				 SaveBatchFileGui("*.bat");
			}
		});		
		
	
//		Row Eleven - Load Batch button
		loadBatButton = new Button(composite, SWT.PUSH);
		loadBatButton.setText("Load .bat");
		loadBatButton.setToolTipText("Load Batch from file");
		GridData gridDataLBB = new GridData();
		gridDataLBB.horizontalAlignment = GridData.BEGINNING;
		loadBatButton.setLayoutData(gridDataLBB);
		loadBatButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e){
				try {
				
					BufferedReader batchReader = new BufferedReader(new FileReader(path.toString() + NewFileGui("*.bat")));
					String command = null;
					command = batchReader.readLine();
					System.out.println("\nloading: " + command);
			  
					try{
									
						String comments = batchReader.readLine();
						while(!comments.trim().startsWith("rem"))
							comments = batchReader.readLine();
						textComment.setText(comments.substring(4));
						checkedComment = true;
						checkBoxComment.setSelection(checkedComment);
						textComment.setEnabled(checkedComment);
					}catch(NullPointerException e1){
						e1.printStackTrace();
					}
		
					int indexStartMa = command.indexOf( "-m" );					
					
					int indexEndMa = command.indexOf( ".ma" );
					if(indexEndMa ==-1)
						indexEndMa = command.indexOf( ".MA" );
					
					int indexStartCell = command.indexOf( "-c" );
					int indexEndCell = command.indexOf( " ", indexStartCell );
					
					int indexStartBmp = command.indexOf( "-b",indexEndCell );
					
					int indexEndBmp = command.indexOf( ".bmp", indexStartBmp );
					if(indexEndBmp ==-1)
						indexEndBmp = command.indexOf( ".BMP", indexStartBmp );
					
					int indexStartMin = command.indexOf( "-l",indexEndBmp );
					int indexEndMin = command.indexOf( " ", indexStartMin );					
					int indexStartMax = command.indexOf( "-u",indexEndMin );
					int indexEndMax = command.indexOf( " ", indexStartMax );
					int indexStartVal = command.indexOf( "-v",indexEndMax );
					int indexEndVal = command.indexOf( ".val", indexStartVal );	
					if(indexEndVal ==-1)
						indexEndVal = command.indexOf( ".VAL", indexStartVal );	
										
					int indexStartPrecision = command.indexOf( "-p",indexEndMin );
					
					//ME REVISED SECTION START
					int indexStartAdv = -1;
					if (indexStartPrecision!=-1) {
						indexStartAdv = command.indexOf( " ", indexStartPrecision);
					} else if (indexStartPrecision==-1) {
						indexStartAdv = command.indexOf( " ", indexEndVal);
					}			
					//ME REVISED SECTION END

					/* //ORIGINAL SECTION START
					if (indexStartPrecision != -1){
						int indexEndPrecision = command.indexOf( " ", indexStartPrecision );
						checkBoxPrecision.setSelection(true);
						textPrecision.setEnabled(true);
						textPrecision.setText(command.substring(indexStartPrecision + 2, indexEndPrecision));
						
						if (indexEndPrecision+ 1< command.length()){
							checkBoxAdv.setSelection(true);
							textAdv.setEnabled(true);
							textAdv.setText(command.substring(indexEndPrecision + 1, command.length()));
						}
					} else {
						checkBoxAdv.setSelection(true);
						textAdv.setEnabled(true); 			
						textAdv.setText(command.substring(indexEndVal + 5, command.length()));
					}
					*/ //ORIGINAL SECTION END
					
					if (indexStartMa != -1) {		
						 textMa.setText(command.substring(indexStartMa + 2, indexEndMa + 3 ).trim());
					}

					if (indexStartBmp != -1) {		
						 textBmp.setText(command.substring(indexStartBmp + 2, indexEndBmp + 4 ).trim());
					}

					if (indexStartVal != -1) {		
						 textVal.setText(command.substring(indexStartVal + 2, indexEndVal + 4 ).trim());
					}

					if (indexStartMin != -1) {		
						 textMin.setText(command.substring(indexStartMin + 2, indexEndMin ).trim());
					}
					
					if (indexStartMax != -1) {		
						 textMax.setText(command.substring(indexStartMax + 2, indexEndMax ).trim());
					}
					
					//ME REVISED SECTION START		
					if (indexStartPrecision != -1) {
						int indexEndPrecision = command.indexOf( " ", indexStartPrecision );
						checkBoxPrecision.setSelection(true);
						textPrecision.setEnabled(true);
						textPrecision.setText(command.substring(indexStartPrecision + 2, indexEndPrecision));
					} else {
						textPrecision.setText("4");
						textPrecision.setEnabled(false);
						checkBoxPrecision.setSelection(false);					
					}
									
					if ( (indexStartAdv!=-1) && ((command.length()-indexStartAdv) > 1) ) {
						//advanced present
						checkBoxAdv.setSelection(true);
						textAdv.setEnabled(true);
						textAdv.setText(command.substring(indexStartAdv+1, command.length()).trim() );							
					} else {
						//advanced not present
						textAdv.setText("");
						checkBoxAdv.setSelection(false);
						textAdv.setEnabled(false);	
					}				
					//ME REVISED SECTION END

					
				}catch (Exception error) {
					  error.printStackTrace();
					  System.out.println("************EXCEPTION: loading batch file******");
				}
			 }
		});
	
		Label errorMessageLabel = new Label(composite, SWT.NONE);
		errorMessageLabel.setFont(parent.getFont());

		return composite;
	}
	
	
	
	/*
	 *creats a lable for dialog box
	 *
	 * @param String message
	 * @param Composite composite 
	 * @param Composite parent 
	 * 
	 * @return Label 
	 **/
	
	private Label createLabel(Composite parent, Composite composite, String message) {
		Label label = new Label(composite, SWT.NONE);
		label.setText(message);
		GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.horizontalSpan=10;		
		label.setLayoutData(gridData);
		label.setFont(parent.getFont());
		return label;
	}	



	/**
		 * @see org.eclipse.jface.window.Window#close()
		 */
	public boolean close() {
		
		if (OK2Close) {
							
			boolean returnValue = super.close();
			return returnValue;
		}
		
		return true;
		
	}
	
	/**
	 * This method takes the fileName and concatenates the digits of n 
	 * [found through testString] to the end of the string [if required].
	 * Edited: by Jing Cao
	 * 
	 * @param fileName
	 * @param integer to be appended
	 * @return String tempFileName [new file name with the number at end]
	 */
	private String setDigit(String fileName, int n) { //beg setDigit
		String tempFileName = new String(fileName);
		String tempString;
		char c;
		int i;

		//Note: the string array only extends from 0 -> (length of string-1)
		for (i = (fileName.length() - 1); i >= 0; i--) { //This for loop will find the location where the number starts.
			c = tempFileName.charAt(i);
			if ((c < '0') || (c > '9'))
				//if the chracter is not a number then break out of the for loop
				break;
			// the value of 'i' will be the index where there is a character [not a number]
		}

		tempFileName = tempFileName.trim();
		//redundant, it gets rid of spaces infront or after the fileName
		tempString = new String(Integer.toString(n));
		if (i < 0) //the fileName consists of all numbers
			return tempString;
		// then return the updated number as the file name!
		//else
		tempFileName = tempFileName.substring(0, i + 1);
		//get the character part of the string
		//note: the +1 is required because substring(beg,end) is "exclusive" of the end.

		return tempFileName.concat(tempString);
		//adds the updated number to the end of the characters and returns the string
	} //end setDigit

	/**
	 * This method takes a file name (stripped off ".bat") and
	 * returns the digits (numbers) at the end as a string.
	 * Returns '0' if there's no number at the end of the file name.
	 * Will return the file name if the string is composed of only numbers.
	 * Edited by Jing Cao.
	 * 
	 * @param fileName
	 * @return number [of the type string]
	 */
	private String testString(String fileName) { //beg testString()

		//Variable Declarations
		String tempString = new String(fileName);
		//redunant but just to make shure that we don't make changes to testString()
		String number = new String(""); //this string stores the number
		int i;
		char c;

		//Note: the string array only extends from 0 -> (length of string-1)
		for (i = (fileName.length() - 1); i >= 0; i--) { //This for loop will find the location where the number starts.
			c = tempString.charAt(i);
			if ((c < '0') || (c > '9'))
				//if the chracter is not a number then break out of the for loop
				break;
			// the value of 'i' will be the index where there is a character [not a number]
		}

		//if statements to set the number string properly
		if (i == (fileName.length() - 1))
			//sets number to 0 if no number found at end of the string.
			number = "0";
		else if (i == -1) //means that the file name is a number!
			number = new String(fileName);
		//then set number to be the file name
		else
			number = tempString.substring(i + 1);
		//creates the number string by chopping off the letters

		number = number.trim();
		//removes any spaces at the beginning or the end of the string

		return number;
	} //end testString()
	
	/*
	 * opens up dialog box for opening/accessing files
	 * 
	 * @param String filter, for viewing files of only type filter(ie: filter = *.ma)
	 */

	private String NewFileGui(String filter) {
		
		
		composite.setEnabled(false);
		simuButton.setEnabled(false);
		doneButton.setEnabled(false);
		OK2Close = false;
		
		FileDialog fileDialog = new FileDialog(parent);
		String[] extention = new String[1];
		extention[0] = filter;
		fileDialog.setFilterPath(path.toString());
		fileDialog.setFilterExtensions(extention);
		fileDialog.open();
		
		composite.setEnabled(true);
		simuButton.setEnabled(true);
		doneButton.setEnabled(true);
		loadBatButton.setEnabled(true);
		saveBatButton.setEnabled(true);
		OK2Close = true;
		
		return fileDialog.getFileName();
	}
	
	
	//this function is used by save.bat to save a file
	//code modified by Sameen Rehman
	private void SaveBatchFileGui(String filter) {
		String[] param = new String[10];
		
		String command = collectParameters();
	    String temp;
		if (command == null){
		
			composite.setEnabled(true);
			doneButton.setEnabled(true);
			simuButton.setEnabled(true);
			saveBatButton.setEnabled(true);
			loadBatButton.setEnabled(true);
			OK2Close = true;
					
			return;
		}
		
		composite.setEnabled(false);
		simuButton.setEnabled(false);
		doneButton.setEnabled(false);
				
		FileDialog fileDialog = new FileDialog(parent, SWT.SAVE);
		String[] extention = new String[1];
		extention[0] = filter;
		fileDialog.setFilterPath(path.toString());
		fileDialog.setFilterExtensions(extention);
		fileDialog.open();
		
		String fileName = fileDialog.getFileName();
		
		try {
			
			if (fileName.endsWith(".bat")) {
				fileName = fileName.substring(0, fileName.length()-4);
			}
			//this code segment makes sure nothing is saved when u cancel the save dialog box
			if(fileName == ""){
			fileDialog.setFilterPath("");
			fileDialog.setFilterExtensions(null);
			composite.setEnabled(true);
			simuButton.setEnabled(true);
			doneButton.setEnabled(true);
			saveBatButton.setEnabled(true);
			loadBatButton.setEnabled(true);
			
			OK2Close = true;
			return;
			}
			
			File existFile = new File(path.toString() + fileName + ".bat");
			boolean cancelled = false;
			if (existFile.exists()) {
				
				IPreferenceStore store = CDBuilderPlugin.getDefault().getPreferenceStore();
				if(store.getInt("batchChoice") == 3) {
					OverwriteGui ovG = new OverwriteGui(new Shell());
					ovG.open();
					if (ovG.buttonPressed() == 0) {
						System.out.print("canceled saving bat");	
						cancelled = true;
						
						composite.setEnabled(true);
						simuButton.setEnabled(true);
						doneButton.setEnabled(true);
						saveBatButton.setEnabled(true);
						loadBatButton.setEnabled(true);
						OK2Close = true;
					}
					if (ovG.buttonPressed() == 2) {
						boolean looking = true;
						
						while(looking) {
							n=(Integer.decode(testString(fileName))).intValue();
							n++;
							
							temp = setDigit(fileName,n);
							File existFileCount = new File(path.toString() + temp+ ".bat");
							System.out.println(path.toString() + temp + ".bat");
							fileName = temp;
							if(!existFileCount.exists()) {
								looking = false;
																	
								MessageBox mbox = new MessageBox(new Shell(),SWT.ICON_INFORMATION | SWT.OK);
								mbox.setText("Saving File");
								mbox.setMessage("The file has been saved as "+ fileName+".bat");
								mbox.open();
																
								composite.setEnabled(true);
								simuButton.setEnabled(true);
								doneButton.setEnabled(true);
								saveBatButton.setEnabled(true);
							    loadBatButton.setEnabled(true);
							}
						}
						if(ovG.noAskAgain() )
							store.setValue("batchChoice", 2);
					}
					if (ovG.buttonPressed() == 1) {
						if(ovG.noAskAgain() )
							store.setValue("batchChoice", 1);
					}
				
				
				}
				else if(store.getInt("batchChoice") == 2) {
					boolean looking = true;
					int n = 0;
					while(looking) {
						n++;
						File existFileCount = new File(path.toString() + fileName + "-" + n + ".bat");
						System.out.println(path.toString() + fileName +"-" + n + ".bat");
						if(!existFileCount.exists()) {
							looking = false;
							fileName = fileName.concat("-" + Integer.toString(n));
						}
					}
				}
				else {
				}
			}
			if (!cancelled) {	
				composite.setEnabled(false);
				simuButton.setEnabled(false);
				doneButton.setEnabled(false);
				OK2Close = false;
				FileWriter batchFile = new FileWriter(path.toString() + fileName + ".bat");
				System.out.println(path.toString() + fileName + ".bat");
				param[0] = "-m" +textMa.getText();
				

				System.out.println(command);
				batchFile.write(command + "\n");
				batchFile.write("rem " + textComment.getText() + "\n");
				batchFile.write("pause");
				batchFile.close();
				composite.setEnabled(true);
				simuButton.setEnabled(true);
				doneButton.setEnabled(true);
				saveBatButton.setEnabled(true);
				loadBatButton.setEnabled(true);
				OK2Close = true;
			}
		}
		catch (Exception e) {
			e.printStackTrace();
		}	
	}
	
}
