/*  
 * v1.0 Edited by Jing and Omair   (05/11/2005)
 * -Fixed FileDialog box; made modal with respect to "this class" so that when it is clicked it stays on top.
 * 
 * -Fixed Save as .bat --> so that now when there's incorrect *.ma filename or if there's no filename
 *  you won't be able to save a *.bat file. 
 * 
 * -Fixed Error in testString()
 * currently the function is setup so that it seeks numbers in the string [no matter where] and contatenates them, adds 1 to the concatenated number and appends the number to the end of the string. 
 * so if you make a file name called 4567i and you overwrite the file [using automatic] you will get a new file called: 4567i4568 when it should be 4567i1
 * Fix: rewrote the function by checking from the back instead of from the front to find the location of a character. Then the numbers [after the char] will be chopped off as a substring and returned.
 * 
 * -Fixed Error in setDigit()
 * When trying to save a *.bat file called "2" and saving another file called "2" using the automatic save will result in either window not responding or nothing happens. 
 * Fix: altered the setDigit() [no more index out of bounds exception] so that when filename is a number it returns a string representation of the newly updated number.
 * 
 * -Fixed Error when closing window
 * Bug: When window is first opened, it will not close when clicking on the "X" (close button at the top right corner of window)
 * Fix: set OK2Close = true by default so that window will close.
 * 
 * -Fixed Error for when canceling browse
 * BUG: When there's a file in the text box [text box filled] beside the browse button and if browse is clicked and you hit cancel to "cancel" file selection. 
 * The file name will be erased from the text box [empty text box] and you will have to reselect the file. 
 * Fix: Check for if the file returned from the file selection is equal to "" [nothing] if so then don't update text box field.
 * 
 * -Fixed error which occurs if the root directory of eclipse has spaces in it
 * Bug: If the root directory of eclipse has spaces in it then this button didn't function at all
 * Fix: We need to enclose IPath objects namely path and internalPath in double quotes by simply concatenating
 * their string representations by "/""
 * 
 * -Fixed error which occurs if any fileNames have a space in them
 * Bug: If the for e.g. say a .val file has spaces in it then an error will occur terminating the function 
 * immediately
 * Fix: All filenames were enclosed in double quotes which was done in collect parameters

 */
 
 
package CELLDEV.buttons;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStreamReader;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.IInputValidator;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.PlatformUI;

import CDBuilder.CDBuilderPlugin;
import CDBuilder.buttons.OverwriteGui;
import CDBuilder.console.ConsoleDocument;
import CDBuilder.console.StreamHandler;


/**
 * @author ssim, Ali Monadi
 *
 * This class is responsible for the popup gui which appears when the SimuAction button  
 * is pressed. When proceed button is pressed, the requested application will run as a separate 
 * process. 
 */
public class GraflogGUI extends Dialog{

	private Shell parent;
	
	private String title;
	private String message1;
	private String message2;
	private String message3;
	private String message4;
	private String message5;
	private String message6;

	private Button simuButton;
	private Button doneButton;
	private Button killButton;
	
	private Button saveSetButton;
	private Button loadSetButton;
	private Button saveBatButton;
	private Button loadBatButton;
	
	private Label colon1;//, colon2, colon3;
	private Button checkBoxOut;
	private Button checkBoxAdv;
	private Button checkBoxTime;
	private Button fileButtonDRW;
	private Button fileButtonOut; 
	private Button checkBoxComment;
	private Button checkBox3D;
	private Button checkBox2D;
	private Button checkBoxV;

	private Text textHour, textMin, textSec, textmilliS;	
	private Text textComment;
	private Text textOut;
	private Text textPalette;
	private Text textAdvanceOverTime;
	private Text textScale;
	private Text textX,textY;
	private Text textAdv;

	private static boolean checkedOut = true;
	private static boolean checkedAdv = false;
	private static boolean checkedComment = false;

	private boolean simuKilled = false;
	private RunSimu rsimu;
	
	private Process simuProcess;
	
	private Label errorMessageLabel;
	private IInputValidator validator;
	private String COLFileName;
	private IPath path;
	private IPath internalPath;
	
	private Composite composite;
	private int n;
	//window will only cose if OK2Close is true. This is set to true automatically at beginning [not set in constructor]
	private boolean OK2Close=true;
	
	
	class conWrite implements Runnable {
		String line;
		public conWrite(String line) {
			this.line = line;
		}
		public void run() {
			ConsoleDocument.getCDOS().set(line);
		}
	}

	class RunSimu extends Thread {
		Dialog d;
		String command;
		public RunSimu(String command, Dialog d) {
			this.command = command;
			this.d = d;
		}
		
		public void run() {
			try {
				
				ConsoleDocument cdos = ConsoleDocument.getCDOS();
				IContainer container = ResourcesPlugin.getWorkspace().getRoot().getContainerForLocation(path);				
							
				FileWriter tempFile = new FileWriter(path.append("tempSimu.bat").toString());
				
				//testing purposes
				tempFile.write("\""+internalPath+"GrafLog\"" + command);  //internalPath is where the cd++.exe is located
				tempFile.close();
				
				simuProcess = Runtime.getRuntime().exec("cmd /c \"" + path.toString() + "tempSimu.bat\"", null, new File(path.toString()));
				StreamHandler outputHandler = new StreamHandler(simuProcess.getInputStream(), "OUTPUT", true);
				StreamHandler errorHandler = new StreamHandler(simuProcess.getErrorStream(), "ERROR", true);            
			
				outputHandler.start();
				errorHandler.start();

				int exitVal = simuProcess.waitFor();

				container.refreshLocal(1,null);
				ResourcesPlugin.getWorkspace().getRoot().getFileForLocation(path.append("tempSimu.bat")).delete(true,null);

				Display.getDefault().syncExec(new Runnable() {
					public void run(){	
						doneButton.setEnabled(true);
						killButton.setEnabled(false);
						simuButton.setEnabled(true);
						d.close();
					}
				});
				
				if (simuKilled) {
							
					Display.getDefault().syncExec(new conWrite("***User has killed the simulation***" + "\n"));
					simuKilled = false;
				
				}
			}
			catch (Throwable T) {
				T.printStackTrace();
			}
		}
	}

	public GraflogGUI(Shell parentShell,IPath path, IPath internalPath,String COLFileName) {
		super(parentShell);
		parent = parentShell;
		this.title = "GrafLog3D";
		message1 = "Pallete file (.col)";
		message2 = "Draw log file (.drw)";
		message3 = "Enter scale(e) and Number of advances over time(s)";
		message4 = "Advanced users Only. Enter desired paramters";
		message5 = "Comments";
		
		this.COLFileName = COLFileName;
		this.internalPath = internalPath;
		this.path = path;
	}
	
	/*
	 * Determines if a text box is left empty and displays error message
	 * @return false of empty true otherwise
	 */
	
	private boolean isEmpty(String parameter,Text textBox){
		String value = textBox.getText().trim();	
		if(value.length()== 0){
				
			MessageBox mbox = new MessageBox(new Shell(),SWT.ICON_ERROR | SWT.OK);
			mbox.setText("Simulation Stopped");
			mbox.setMessage(parameter+" is not entered");
			mbox.open();
			
			try {
				mbox.wait();
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
			doneButton.setEnabled(true);
			killButton.setEnabled(false);
			simuButton.setEnabled(true);
			return true;
		}
		return false;
	}
	
	
	/*
	 * checks to see if the given string matches a file name found in the project directory
	 * 
	 * @param String file, name of file 
	 * @param String errMsg, message to be displayed IF file was not found
	 */	
	
	private boolean checkFileExistence(String file, String errMsg){
		
		IWorkspaceRoot root = (IWorkspaceRoot)PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().getInput();
		IResource[] files;
		IFile fileHandle = null;
	
		try {
			files = root.getContainerForLocation(path).members();
	
			for(int i=0;i<files.length;i++){
				
				if(files[i]instanceof IFile && files[i].getName().equalsIgnoreCase(file)){
					fileHandle = (IFile) files[i];
					break;
				}
			}
			if (fileHandle == null){
	
				MessageBox mbox = new MessageBox(new Shell(), SWT.ICON_ERROR | SWT.OK);
				mbox.setText("Simulation Stopped");
				mbox.setMessage(errMsg);
				mbox.open();
				try {
					mbox.wait();
				} catch (InterruptedException e) {
					e.printStackTrace();
				}			
				return false;
			}
			return true;
				
		} catch (CoreException e) {
			MessageBox mbox = new MessageBox(new Shell(), SWT.ICON_ERROR | SWT.OK);
			mbox.setText("Simulation Stopped");
			mbox.setMessage("Error occured while accessing file");
			mbox.open();
			try {
				mbox.wait();
			} catch (InterruptedException ex) {
				ex.printStackTrace();
			}		
			e.printStackTrace();
			return false;	
		}			
	}
	


	/*
	 * This function will check to see if the file name entered in the Text field contains
	 * the correct extention. If it does not have an extention it will concate the extention to 
	 * the end of the file name 
	 * 
	 * @param Text name, the text box value containing the file name
	 * @param String extention, the extention the name shoule have
	 * @return a string containing the file name with the proper extention
	 */
	
	private String checkExtention(Text name , String extention){
		String fileName = name.getText().trim();
		
		if(!fileName.endsWith(extention) && !fileName.endsWith(extention.toUpperCase()))
			return fileName.concat(extention); 
		return fileName;
	}	
	
	
	/*
	 * This function will record all the parameters entered into a string
	 *
	 * @return String will contain all parameters. if there is an error found NULL will be returned
	 */	
	private String collectParameters(){
		String[] param = new String[10];
		String combine = "";
		
		if(isEmpty("color palette file",textPalette)|| isEmpty("s value",textAdvanceOverTime) 
		             || isEmpty("e value",textScale)|| isEmpty("DrawLog file",textOut)){
			return null;
		}

		if(!checkFileExistence(checkExtention(textPalette , ".col"), "palette file not found")
		 ||!checkFileExistence(checkExtention(textOut , ".drw"), "Drawlog file not found")){
			return null;
		}	
		
		if (checkBoxV.getSelection() == true){
			param[0]= "-v";
		}

		if (checkBox3D.getSelection() == true){
			param[1] = "-3";
						
		}else if (checkBox2D.getSelection() == true){
			param[1] = "-2";
			param[2] = "-s"+ textAdvanceOverTime.getText().trim();
			
		}else{
			MessageBox mbox = new MessageBox(new Shell(), SWT.ICON_ERROR | SWT.OK);
			mbox.setText("Error");
			mbox.setMessage("Must force 2D or 3D Graphics");
			mbox.open();
			try {
				mbox.wait();
			} catch (InterruptedException ex) {
				ex.printStackTrace();
			}		
			return null;
		}
		
		param[3] = "-e" + textScale.getText().trim();
		param[4] = "-f\"" + checkExtention(textOut,".drw")+"\""  ;
		param[5] = "-c\"" + checkExtention(textPalette,".col")+"\"" ;

		if(textAdv.isEnabled()) {
			String advValue= textAdv.getText().trim();
			if(isEmpty("Advance parameter",textAdv)){
				return null;
			}
			param[6] = advValue;
		}
			
		for (int n=0;  n < 7; n++ ) {
			if (param[n] != null)
				combine = combine + " " + param[n];
		}
		System.out.println(combine);
		return combine;
	}
	
	
	
	
	protected void buttonPressed(int buttonId) {
		String[] param = new String[10];
		if (buttonId == IDialogConstants.CLOSE_ID){
		    OK2Close = true;
			close();
		}
		if (buttonId == IDialogConstants.PROCEED_ID) {
			
			String command = collectParameters();
			if(command == null){
				return;
			}
			System.out.println(command);
			rsimu = new RunSimu(command,this);
			rsimu.start();
			close();
		} 
		super.buttonPressed(buttonId);
		
	}


	protected void configureShell(Shell shell) {
		super.configureShell(shell);
		if (title != null)
			shell.setText(title);
	}
	
	
	/*
	 *creats a lable for dialog box
	 *
	 * @param String message
	 * @param Composite composite 
	 * @param Composite parent 
	 * 
	 * @return Label 
	 **/
	
	private Label createLabel(Composite parent, Composite composite, String message) {
		Label label = new Label(composite, SWT.NONE);
		label.setText(message);
		GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.horizontalSpan=10;		
		label.setLayoutData(gridData);
		label.setFont(parent.getFont());
		return label;
	}
	
	
	
	protected void createButtonsForButtonBar(Composite parent) {
		// create OK and Cancel buttons by default
		simuButton = createButton(parent, IDialogConstants.PROCEED_ID, IDialogConstants.PROCEED_LABEL, true);
		simuButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e){
				killButton.setEnabled(true);
				simuButton.setEnabled(false);
				doneButton.setEnabled(false);
			}
		});
		doneButton = createButton(parent, IDialogConstants.CLOSE_ID, IDialogConstants.CLOSE_LABEL, true);
		killButton = createButton(parent, IDialogConstants.STOP_ID, "Kill", true);
		killButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e){
				simuProcess.destroy();
				try {
					Process lookProcess = Runtime.getRuntime().exec("ps");
					BufferedReader bf = new BufferedReader(new InputStreamReader(lookProcess.getInputStream()));
					String msg = null;
					while((msg = bf.readLine())!= null){
						if (msg.indexOf(path.toString().substring(2))!= -1){
							msg = msg.substring(2).trim();
							Process killProcess=Runtime.getRuntime().exec("kill -9 " + msg.substring(0,msg.indexOf(" ")));
						}
					}
				} catch (IOException e1) {
					e1.printStackTrace();
				}
				doneButton.setEnabled(true);
				killButton.setEnabled(false);
				simuButton.setEnabled(true);
				simuKilled = true;
			}
		});
		
		killButton.setEnabled(false);
		//do this here because setting the text will set enablement on the ok button
		textPalette.setFocus();
	
		
	}
	
	protected Control createDialogArea(Composite parent) {
		// create composite( which is a control that can contain other controls!)
		composite = (Composite)super.createDialogArea(parent);
		GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 10;
		composite.setLayout(gridLayout);

        //Row One - Should be text description of ma text box
		Label label = createLabel(parent, composite, message1);

		// Row two - Text Box for the drw file and file button
		textPalette = new Text(composite, SWT.SINGLE | SWT.BORDER);
		if (COLFileName != null)
			textPalette.setText(COLFileName);

		
		Color c = new Color(null,255,255,255);
		textPalette.setBackground(c);
		GridData gridData = new GridData();
		gridData.horizontalSpan=9;
		gridData.horizontalAlignment = GridData.CENTER;
		gridData.grabExcessHorizontalSpace = true;
		gridData.widthHint = convertHorizontalDLUsToPixels(IDialogConstants.MINIMUM_MESSAGE_AREA_WIDTH);
		textPalette.setLayoutData(gridData);
		Button fileButtonDRW = new Button(composite, SWT.PUSH);
		fileButtonDRW.setText("Browse");
		fileButtonDRW.setToolTipText("Select .col File");
		GridData gridDataFBMa = new GridData();
		gridDataFBMa.horizontalAlignment = GridData.BEGINNING;
		fileButtonDRW.setLayoutData(gridDataFBMa);
		fileButtonDRW.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e){
				composite.setEnabled(false);
				simuButton.setEnabled(false);
				doneButton.setEnabled(false);
				saveBatButton.setEnabled(false);
				loadBatButton.setEnabled(false);
				OK2Close = false;
				String fileName = NewFileGui("*.col");
				if (!fileName.equals("")) 
				textPalette.setText( fileName);
			}
		});


		//Row Five - Text Description of the Out text box
		label = createLabel(parent, composite, message2);


		//Row Six - The  text box for out

		textOut = new Text(composite, SWT.SINGLE | SWT.BORDER);
		gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.horizontalSpan=9;
		textOut.setLayoutData(gridData);
		textOut.setEnabled(checkedOut);
		fileButtonOut = new Button(composite, SWT.PUSH);
		fileButtonOut.setText("Browse");
		fileButtonOut.setToolTipText("Select Output File");
		GridData gridDataFBOut = new GridData(SWT.WRAP);
		gridDataFBOut.horizontalAlignment = GridData.BEGINNING;
		fileButtonOut.setLayoutData(gridDataFBOut);
		fileButtonOut.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e){
			composite.setEnabled(false);
			simuButton.setEnabled(false);
			doneButton.setEnabled(false);
			saveBatButton.setEnabled(false);
			loadBatButton.setEnabled(false);
			OK2Close = false;
			String fileName = NewFileGui("*.drw");
			if (!fileName.equals(""))
			textOut.setText( fileName);
			}
		});
		fileButtonOut.setEnabled(checkedOut);
		
		

		//Row Seven - A text description of the couple label

		label = createLabel(parent, composite, "Force 3D Graphics");
		checkBox3D = new Button(composite, SWT.CHECK);
		gridData = new GridData();
		gridData.horizontalAlignment = GridData.BEGINNING;
		checkBox3D.setLayoutData(gridData);
		checkBox3D.setSelection(false);

		checkBox3D.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e){
				if(checkBox3D.getSelection() == true){

					checkBox2D.setEnabled(false);
					textAdvanceOverTime.setEnabled(false);
					
				}else if(checkBox3D.getSelection() == false){

					checkBox2D.setEnabled(true);
					textAdvanceOverTime.setEnabled(true);
				
				}
			}
		});
	
		label = createLabel(parent, composite, "Force 2D Graphics:");
		checkBox2D = new Button(composite, SWT.CHECK);
		gridData = new GridData();	
		gridData.horizontalAlignment = GridData.FILL_BOTH;	
		checkBox2D.setLayoutData(gridData);
		checkBox2D.setSelection(false);

		checkBox2D.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e){
				if(checkBox2D.getSelection() == true){

					checkBox3D.setEnabled(false);
					
					
				}else if(checkBox2D.getSelection() == false){

					checkBox3D.setEnabled(true);
					
				
				}
			}
		});
		
		
		label = createLabel(parent, composite, "View Lattice");
		checkBoxV = new Button(composite, SWT.CHECK);
		gridData = new GridData();	
		gridData.horizontalAlignment = GridData.BEGINNING;
		checkBoxV.setLayoutData(gridData);
		checkBoxV.setSelection(false);

		checkBoxV.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e){
				if(checkBoxV.getSelection() == true){

					
				}else if(checkBoxV.getSelection() == false){


				
				}
			}
		});



		//Row Nine - Text description of the scale and number of advances over the time, text box
		label = createLabel(parent, composite, message3);

		//Row ten -  and text box for time
		colon1 = new Label(composite,SWT.NONE);
		colon1.setText("e:");
			
		textScale = new Text(composite, SWT.SINGLE | SWT.BORDER);
		textScale.setTextLimit(12);
		
		GridData gridData1 = new GridData();
		gridData1.horizontalAlignment = GridData.BEGINNING;

		gridData1.widthHint = 85;
		gridData1.heightHint = 13;
		textScale.setLayoutData (gridData1);
		textScale.setEnabled(true);
		textScale.setToolTipText("e value:");
		textScale.setText("0");
		textScale.addKeyListener(new KeyListener(){
			public void keyPressed(KeyEvent e) {
				try{
					Integer value = new Integer(textScale.getText());
					if(value.intValue() < 0){
						textScale.setText("0");
					}
					if(e.keyCode == SWT.ARROW_UP){
						value = new Integer(value.intValue()+1);
						textScale.setText(value.toString());
						
					}else if(e.keyCode == SWT.ARROW_DOWN){
						value = new Integer(value.intValue()-1);
						textScale.setText(value.toString());

					}
					if(value.intValue() < 0){
						textScale.setText("0");
					}
									
				}catch(Exception e1){
					textScale.setText("");
				}
			}

			public void keyReleased(KeyEvent e) {
						
			}
		});		
			
		colon1 = new Label(composite,SWT.NONE);
		colon1.setText("s:");
		textAdvanceOverTime = new Text(composite, SWT.SINGLE | SWT.BORDER);
		textAdvanceOverTime.setTextLimit(12);
		gridData1 = new GridData();
		gridData1.horizontalAlignment = GridData.BEGINNING;
		gridData1.widthHint = 85;
		//gridData1.heightHint = 13;
		textAdvanceOverTime.setLayoutData (gridData1);
		textAdvanceOverTime.setEnabled(true);
		textAdvanceOverTime.setToolTipText("S value:");
		textAdvanceOverTime.setText("0");
		textAdvanceOverTime.addKeyListener(new KeyListener(){
			public void keyPressed(KeyEvent e) {
				try{
					Integer value= new Integer(textAdvanceOverTime.getText());
	
					if(e.keyCode == SWT.ARROW_UP){
						value = new Integer(value.intValue()+1);
						textAdvanceOverTime.setText(value.toString());
						
					}else if(e.keyCode == SWT.ARROW_DOWN){
						value = new Integer(value.intValue()-1);
						textAdvanceOverTime.setText(value.toString());
					}
					if(value.intValue() < 0){
						textAdvanceOverTime.setText("0");
					}						
								
				}catch(Exception e1){
					textAdvanceOverTime.setText("");
				}
			}
				public void keyReleased(KeyEvent e) {
				
			}
		});	


		//Row Eleven - Text Discription of Advanced					
		Label label6 = new Label(composite, SWT.WRAP);
		label6.setText(message4);
		GridData gridData16 = new GridData();
		gridData16.horizontalAlignment = GridData.BEGINNING;
		gridData16.horizontalSpan = 10;		
		label6.setLayoutData(gridData16);
		label6.setFont(parent.getFont());

		//Row Tweleve - The check box and text box for advanced		
		checkBoxAdv = new Button(composite, SWT.CHECK);
		GridData gridData17 = new GridData();
		gridData17.horizontalAlignment = GridData.BEGINNING;
		checkBoxAdv.setLayoutData(gridData17);
		checkBoxAdv.setSelection(checkedAdv);
		checkBoxAdv.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e){
				if(textAdv.getEnabled() == true) { 
					textAdv.setEnabled(false);
					checkedAdv = false;
				}	
				else { 
					textAdv.setEnabled(true);
					checkedAdv = true;
				}
			}
		});
		
		textAdv = new Text(composite, SWT.SINGLE | SWT.BORDER);
		GridData gridData18 = new GridData();
		gridData18.horizontalAlignment = GridData.FILL;
		gridData18.grabExcessHorizontalSpace = true;
		gridData18.horizontalSpan = 9;
		textAdv.setLayoutData(gridData18);
		textAdv.setEnabled(checkedAdv);
		
		
//		Row Ten - Text description of the Comment text box
		Label label8 = new Label(composite, SWT.WRAP);
		label8.setText(message5);
		GridData gridData19 = new GridData();
		gridData19.horizontalAlignment = GridData.BEGINNING;
		gridData19.horizontalSpan = 10;		
		label8.setLayoutData(gridData19);
		label8.setFont(parent.getFont());
		
//		Row eight - The check box and text box for comment
		checkBoxComment = new Button(composite, SWT.CHECK);
		GridData gridData20 = new GridData();
		gridData20.horizontalAlignment = GridData.BEGINNING;
		checkBoxComment.setLayoutData(gridData20);
		checkBoxComment.setSelection(false);
		checkBoxComment.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e){
				if(textComment.getEnabled() == true) { 
					textComment.setEnabled(false);
					checkedComment = false;	
				}
				else { 
					textComment.setEnabled(true);
					checkedComment = true;
				}
			}
		});			
		textComment= new Text(composite, SWT.SINGLE | SWT.BORDER);
		GridData gridData21 = new GridData();
		gridData21.horizontalSpan = 9;
		gridData21.horizontalAlignment = GridData.FILL;
		gridData21.grabExcessHorizontalSpace = true;
		textComment.setLayoutData(gridData21);
		textComment.setEnabled(false);
		
		
		

		//Row Sixteen - Save as Batch button
		saveBatButton = new Button(composite, SWT.PUSH);
		saveBatButton.setText("Save as .bat");
		saveBatButton.setToolTipText("Save Settings to batch file");
		GridData gridDataSBB= new GridData();
		gridDataSBB.horizontalAlignment = GridData.END;
		gridDataSBB.horizontalSpan = 9;	
		saveBatButton.setLayoutData(gridDataSBB);
		saveBatButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e){
				composite.setEnabled(false);
				simuButton.setEnabled(false);
				doneButton.setEnabled(false);
				saveBatButton.setEnabled(false);
				loadBatButton.setEnabled(false);
				OK2Close = false;
				SaveBatchFileGui("*.bat");
			}
		});	
		
//		Row Fifteen - Load Batch button
			  loadBatButton = new Button(composite, SWT.PUSH);
			  loadBatButton.setText("Load .bat");
			  loadBatButton.setToolTipText("Load Batch from file");
			  GridData gridDataLBB = new GridData();
			  gridDataLBB.horizontalAlignment = GridData.BEGINNING;
			  loadBatButton.setLayoutData(gridDataLBB);
			  loadBatButton.addSelectionListener(new SelectionAdapter() {
				  public void widgetSelected(SelectionEvent e){
					try {
					
						BufferedReader batchReader = new BufferedReader(new FileReader(path.toString() + NewFileGui("*.bat")));
						String command = null;
						command = batchReader.readLine();
						System.out.println("\nloading: " + command);
						  
						try{
									
							String comments = batchReader.readLine();
							while(!comments.trim().startsWith("rem"))
								comments = batchReader.readLine();
							textComment.setText(comments.substring(4));
							checkedComment = true;
							checkBoxComment.setSelection(checkedComment);
							textComment.setEnabled(checkedComment);
						}catch(NullPointerException e1){
							e1.printStackTrace();
						}
						int indexStartS = -1;
						int indexEndS = -1;
						int indexStartV = command.indexOf("-v");
						int indexStart2D = command.indexOf("-2");
						int indexStart3D = command.indexOf("-3");
						
						if (indexStart2D != -1){
							indexStartS = command.indexOf("-s");
							indexEndS = command.indexOf(" ", indexStartS+1);
						}
						int indexStartE = command.indexOf("-e");
						int indexEndE = command.indexOf(" ", indexStartE+1);
						

						int indexStartdrw = command.indexOf("-f");
						
						int indexEnddrw = command.indexOf(".drw");
						if (indexEnddrw == -1)
							indexEnddrw = command.indexOf(".DRW");
							
						int indexStartCOL = command.indexOf("-c");
						
						int indexEndCOL = command.indexOf(".col");
						if (indexEndCOL == -1)
							indexEndCOL = command.indexOf(".COL");
						
						if (indexStart2D != -1){
							checkBox2D.setSelection(true);
							textAdvanceOverTime.setText( command.substring(indexStartS + 2, indexEndS ).trim() );
							checkBox3D.setEnabled(false);
						
						}else if(indexStart3D != -1){
							checkBox3D.setSelection(true);
							checkBox2D.setEnabled(false);
							textAdvanceOverTime.setEnabled(false);
						}
						
						if (indexStartE != -1){
							textScale.setText( command.substring(indexStartE + 2, indexEndE ).trim() );
						}
						
						if (indexEndCOL+4 < command.length()){
							checkBoxAdv.setSelection(true);
							textAdv.setEnabled(true);	
							textAdv.setText( command.substring(indexEndCOL+4, command.length() ).trim() );
						}	
						
						if (indexStartV != -1 ){
							checkBoxV.setSelection(true);
						}
						
						if (indexStartdrw != -1){
							textOut.setText( command.substring(indexStartdrw + 2, indexEnddrw + 4  ).trim() );
							
						}else{
							textOut.setText( "" );
						}
						
						if (indexStartCOL != -1){
							textPalette.setText( command.substring(indexStartCOL + 2, indexEndCOL + 4  ).trim() );
							
						}else{
							textPalette.setText( "" );
						}
						int indexAdv;

				


						

			
					  }catch (Exception error) {
						  error.printStackTrace();
						  System.out.println("************EXCEPTION: opening file******");
					  }
				  }
			  });
	
		errorMessageLabel = new Label(composite, SWT.NONE);
		errorMessageLabel.setFont(parent.getFont());

		return composite;
	}
	
	/**
		 * @see org.eclipse.jface.window.Window#close()
		 */
	public boolean close() {
		
		if (OK2Close) {
							
			boolean returnValue = super.close();
			return returnValue;
		}
		
		return true;
		
	}
	
	//This method takes the fileName returned by testString, 
	//strips off the digits at the end
	//and concatenates the digits of n to the end of the string.
	private String setDigit(String fileName, int n){
		String test;
		char c;
		boolean done = false;
		test = new String(fileName);
		c=test.charAt(fileName.length()-1);
			
		while(!done){
		if((c>='0') && (c<='9')){
			test = test.substring(0,test.length()-1);
			c=test.charAt(test.length()-1);
		}
		else{
			done = true;		
		}
						
		}
				
		test= test.trim();
		test= test.concat(Integer.toString(n));	
		
	return test;
	}
	
	
		
	//this method takes a filename(stripped off ".bat") and
	// returns the digits at the end as a string
	private String testString(String fileName){
		String test;
		String number=new String("");
		char c;
		
		boolean done = false;
		
		test = new String(fileName);
		c = test.charAt(0);
		int counter = test.length();
		
		while(!done){
			counter--;
			if((c>='0') && (c<='9')){
			number = number.concat(String.valueOf(c));					
			}
			if(counter!=0){
			test = test.substring(1, test.length());
			c = test.charAt(0);
			}
			if(counter == 0){done=true;}
		}
		if(number.equals(""))number= "0";
		number = number.trim();
		return number;
	}
	
	
	//this function is used by load.bat to load a new file. 
	private String NewFileGui(String filter) {
		
		composite.setEnabled(false);
		simuButton.setEnabled(false);
		doneButton.setEnabled(false);
		OK2Close = false;
		
		FileDialog fileDialog = new FileDialog(parent);
		String[] extention = new String[1];
		extention[0] = filter;
		fileDialog.setFilterPath(path.toString());
		fileDialog.setFilterExtensions(extention);
		fileDialog.open();
		
		composite.setEnabled(true);
		simuButton.setEnabled(true);
		doneButton.setEnabled(true);
		loadBatButton.setEnabled(true);
		saveBatButton.setEnabled(true);
		OK2Close = true;
		return fileDialog.getFileName();
	}
	
	
	
	/*
	 * This function is responsible for saving a batch file
	 */
	private void SaveBatchFileGui(String filter) {
		String[] param = new String[10];
		String Command = collectParameters();
		String temp;
		if (Command == null){
					composite.setEnabled(true);
					doneButton.setEnabled(true);
					simuButton.setEnabled(true);
					saveBatButton.setEnabled(true);
					loadBatButton.setEnabled(true);
					OK2Close = true;
					return;
		}
		
		composite.setEnabled(false);
		simuButton.setEnabled(false);
		doneButton.setEnabled(false);
		
		FileDialog fileDialog = new FileDialog(parent, SWT.SAVE);
		String[] extention = new String[1];
		extention[0] = filter;
		fileDialog.setFilterPath(path.toString());
		fileDialog.setFilterExtensions(extention);
		fileDialog.open();
		
		String fileName = fileDialog.getFileName();
		
		try {
			
			if (fileName.endsWith(".bat")) {
				fileName = fileName.substring(0, fileName.length()-4);
			}
			
			if(fileName == ""){
				fileDialog.setFilterPath("");
				fileDialog.setFilterExtensions(null);
				composite.setEnabled(true);
				simuButton.setEnabled(true);
				doneButton.setEnabled(true);
				saveBatButton.setEnabled(true);
				loadBatButton.setEnabled(true);
				OK2Close = true;
				return;
			}
			
			File existFile = new File(path.toString() + fileName + ".bat");
			boolean cancelled = false;
			if (existFile.exists()) {
				
				IPreferenceStore store = CDBuilderPlugin.getDefault().getPreferenceStore();
				if(store.getInt("batchChoice") == 3) {
					OverwriteGui ovG = new OverwriteGui(new Shell());
					ovG.open();
					if (ovG.buttonPressed() == 0) {
						System.out.print("canceled saving bat");	
						cancelled = true;
						composite.setEnabled(true);
						simuButton.setEnabled(true);
						doneButton.setEnabled(true);
						saveBatButton.setEnabled(true);
						loadBatButton.setEnabled(true);
						OK2Close = true;
					}
					
					if (ovG.buttonPressed() == 2) {
						boolean looking = true;
						int n = 0;
						while(looking) {
							n=(Integer.decode(testString(fileName))).intValue();
							n++;
							
							temp = setDigit(fileName,n);
							File existFileCount = new File(path.toString() + temp+ ".bat");
							System.out.println(path.toString() + temp + ".bat");
							fileName = temp;
							if(!existFileCount.exists()) {
								looking = false;
																	
								MessageBox mbox = new MessageBox(new Shell(),SWT.ICON_INFORMATION | SWT.OK);
								mbox.setText("Saving File");
								mbox.setMessage("The file has been saved as "+ fileName+".bat");
								mbox.open();
																
								composite.setEnabled(true);
								simuButton.setEnabled(true);
								doneButton.setEnabled(true);
								saveBatButton.setEnabled(true);
								loadBatButton.setEnabled(true);
							}
						}
						if(ovG.noAskAgain() )
							store.setValue("batchChoice", 2);
					}
					if (ovG.buttonPressed() == 1) {
						if(ovG.noAskAgain() )
							store.setValue("batchChoice", 1);
					}

				}
				else if(store.getInt("batchChoice") == 2) {
					boolean looking = true;
					int n = 0;
					while(looking) {
						n++;
						File existFileCount = new File(path.toString() + fileName + "-" + n + ".bat");
						System.out.println(path.toString() + fileName +"-" + n + ".bat");
						if(!existFileCount.exists()) {
							looking = false;
							fileName = fileName.concat("-" + Integer.toString(n));
						}
					}
				}
				else {
				}
			}
			if (!cancelled) {
				composite.setEnabled(false);
				simuButton.setEnabled(false);
				doneButton.setEnabled(false);
				OK2Close = false;
					
				FileWriter batchFile = new FileWriter(path.toString() + fileName + ".bat");
				System.out.println(path.toString() + fileName + ".bat");

				String command = collectParameters();
				if (command == null){
					batchFile.close();
					return;
				}
				System.out.println(command);

				batchFile.write(command + "\n");
				batchFile.write("rem " + textComment.getText() + "\n");
				batchFile.write("pause");
				batchFile.close();
				composite.setEnabled(true);
				simuButton.setEnabled(true);
				doneButton.setEnabled(true);
				saveBatButton.setEnabled(true);
				loadBatButton.setEnabled(true);
				OK2Close = true;
			
			}
		}
		catch (Exception e) {
			e.printStackTrace();
		}	
	}
}
