/*  
 * v1.0 Edited by Jing and Omair   (05/11/2005)
 * -Fixed FileDialog box; made modal with respect to "this class" so that when it is clicked it stays on top.
 * 
 * -Fixed Save as .bat --> so that now when there's incorrect *.ma filename or if there's no filename
 *  you won't be able to save a *.bat file. 
 * 
 * -Fixed Error in testString()
 * currently the function is setup so that it seeks numbers in the string [no matter where] and contatenates them, adds 1 to the concatenated number and appends the number to the end of the string. 
 * so if you make a file name called 4567i and you overwrite the file [using automatic] you will get a new file called: 4567i4568 when it should be 4567i1
 * Fix: rewrote the function by checking from the back instead of from the front to find the location of a character. Then the numbers [after the char] will be chopped off as a substring and returned.
 * 
 * -Fixed Error in setDigit()
 * When trying to save a *.bat file called "2" and saving another file called "2" using the automatic save will result in either window not responding or nothing happens. 
 * Fix: altered the setDigit() [no more index out of bounds exception] so that when filename is a number it returns a string representation of the newly updated number.
 * 
 * -Fixed Error when closing window
 * Bug: When window is first opened, it will not close when clicking on the "X" (close button at the top right corner of window)
 * Fix: set OK2Close = true by default so that window will close.
 * 
 * -Fixed Error for when canceling browse
 * BUG: When there's a file in the text box [text box filled] beside the browse button and if browse is clicked and you hit cancel to "cancel" file selection. 
 * The file name will be erased from the text box [empty text box] and you will have to reselect the file. 
 * Fix: Check for if the file returned from the file selection is equal to "" [nothing] if so then don't update text box field.
 * 
 * -Fixed Error for when there's a space in file name
 * Fix: Quotes are added to file paths
 * 
 * - Fixed the bug for browsing for files that contains spaces
 * Bug: Files with spaces will not run.
 * Fix: In collectParameters all file names are enclosed in quotes.
 */
 
package CELLDEV.buttons;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.PlatformUI;

import CDBuilder.CDBuilderPlugin;
import CDBuilder.buttons.OverwriteGui;
import CDBuilder.console.ConsoleDocument;
import CDBuilder.console.StreamHandler;


/**
 * @author Ali Monadi
 *
 * This class is responsible for the popup gui which appears when the MakeRand Button is pressed.  
 * The requested application will run as a separate process when proceed button is pressed
 * 
 */

public class MakerandGUI extends Dialog{

	private Shell parent;
	
	private String title;
	private String message1;
	private String message2;
	private String message3;
	private String message4;
	private String message5;

	private String valFile;
	private String coupleMessage;
	
	private Button simuButton;
	private Button doneButton;
	private Button killButton;
	
	private Button saveSetButton;
	private Button loadSetButton;
	private Button saveBatButton;
	private Button loadBatButton;
	private Button checkBoxAdv;
	private Button checkBoxComment;
	private Button checkBoxs0;
	private Button checkBoxs1;
	private Button checkBoxs2;
	private Button checkBoxs3;

	private Combo coupleCombo;

	private Text textMa;
	private Text textVal;
	private Text textAdv;
	private Text textComment;
	private Text nValue;
	
	private static boolean checkedAdv = false;
	private static boolean checkedComment = false;

	private boolean simuKilled = false;
	private RunSimu rsimu;
	
	private Process simuProcess;
	
	private IPath path;
	private IPath internalPath;
	private String maFileName;
	
	private Composite composite;
	private int n;
	//window will only cose if OK2Close is true. This is set to true automatically at beginning [not set in constructor]
	private boolean OK2Close=true;
	
	class conWrite implements Runnable {
		String line;
		public conWrite(String line) {
			this.line = line;
		}
		public void run() {
			ConsoleDocument.getCDOS().set(line);
		}
	}

	class RunSimu extends Thread {
		Dialog d;
		String command;
		public RunSimu(String command, Dialog d) {
			this.command = command;
			this.d = d;
		}
		
		public void run() {
			try {
				
				ConsoleDocument cdos = ConsoleDocument.getCDOS();
				IContainer container = ResourcesPlugin.getWorkspace().getRoot().getContainerForLocation(path);				
							
				FileWriter tempFile = new FileWriter(path.append("tempSimu.bat").toString());
				
				//testing purposes
				System.out.println("command: " + command);
				tempFile.write("\"" + internalPath + "makerand\"" + command);  //internalPath is where the cd++.exe is located
				tempFile.close();
				
				simuProcess = Runtime.getRuntime().exec("cmd /c \"" + path.toString() + "tempSimu.bat\"", null, new File(path.toString()));
				StreamHandler outputHandler = new StreamHandler(simuProcess.getInputStream(), "OUTPUT", true);
				StreamHandler errorHandler = new StreamHandler(simuProcess.getErrorStream(), "ERROR", true);            
			
				outputHandler.start();
				errorHandler.start();

				int exitVal = simuProcess.waitFor();
				Display.getDefault().syncExec(new conWrite("Simulation complete" + "\n"));
				container.refreshLocal(1,null);
				ResourcesPlugin.getWorkspace().getRoot().getFileForLocation(path.append("tempSimu.bat")).delete(true,null);

				Display.getDefault().syncExec(new Runnable() {
					public void run(){	
						doneButton.setEnabled(true);
						killButton.setEnabled(false);
						simuButton.setEnabled(true);
						d.close();
					}
				});
				
				if (simuKilled) {
							
					Display.getDefault().syncExec(new conWrite("***User has killed the simulation***" + "\n"));
					simuKilled = false;
				
				}
			}
			catch (Throwable T) {
				T.printStackTrace();
			}
		}
	}// end of class RunSimu



	/*
	 * constructor
	 */
	public MakerandGUI(Shell parentShell,IPath path, IPath internalPath, String maFileName) {
		super(parentShell);
		parent = parentShell;
		this.title = "Create VAL file with random initial state";
		message1 = "Model file (.ma)";
		coupleMessage = "Cellular Model name";
		message4 = "Advanced Users Only. Enter desired parameters:";
		message5 = "Comments";
		valFile =  "Val file (.val)";
		
		this.maFileName = maFileName;
		this.internalPath = internalPath;
		this.path = path;
	}
	
	
	/*
	 * Determines if a text box is left empty and displays error message
	 * @return false of empty true otherwise
	 */
	
	private boolean isEmpty(String parameter,Text textBox){
		String value = textBox.getText().trim();	
		if(value.length()== 0){
				
			MessageBox mbox = new MessageBox(new Shell(),SWT.ICON_ERROR | SWT.OK);
			mbox.setText("Simulation Stopped");
			mbox.setMessage(parameter+" is not entered");
			mbox.open();
			
			doneButton.setEnabled(true);
			killButton.setEnabled(false);
			simuButton.setEnabled(true);
			return true;
		}
		return false;
	}
	
	
	/*
	 * checks to see if the given string matches a file name found in the project directory
	 * 
	 * @param String file, name of file 
	 * @param String errMsg, message to be displayed IF file was not found
	 */	

	private boolean checkFileExistence(String file, String errMsg){
	
		IWorkspaceRoot root = (IWorkspaceRoot)PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().getInput();
		IResource[] files;
		IFile fileHandle = null;

		try {
			files = root.getContainerForLocation(path).members();

			for(int i=0;i<files.length;i++){
			
				if(files[i]instanceof IFile && files[i].getName().equalsIgnoreCase(file)){
					fileHandle = (IFile) files[i];
					break;
				}
			}
			if (fileHandle == null){

				MessageBox mbox = new MessageBox(new Shell(), SWT.ICON_ERROR | SWT.OK);
				mbox.setText("Simulation Stopped");
				mbox.setMessage(errMsg);
				mbox.open();
						
				return false;
			}
			return true;
			
		} catch (CoreException e) {
			MessageBox mbox = new MessageBox(new Shell(), SWT.ICON_ERROR | SWT.OK);
			mbox.setText("Simulation Stopped");
			mbox.setMessage("Error occured while accessing file");
			mbox.open();
				
			e.printStackTrace();
			return false;	
		}			
	}
	
	
	/*
	 * This function will check to see if the file name entered in the 
	 * 
	 * @param Text name, the text box value containing the file name
	 * @param String extention, the extention the name shoule have
	 * @return a string containing the file name with the proper extention
	 */
	
	public String checkExtention(Text name , String extention){
		String fileName = name.getText().trim();
		
		if(!fileName.endsWith(extention) && !fileName.endsWith(extention.toUpperCase()))
			return fileName.concat(extention); 
		return fileName;
	}
	
	
	/*
	 * This function will record all the parameters entered into a string
	 *
	 * @return String will contain all parameters. if there is an error found NULL will be returned
	 */
	public String collectParameters(){
		String[] param = new String[5];

		if(isEmpty("Model file",textMa)){
			return null;
		}
		
		if( !checkFileExistence(checkExtention(textMa,".ma"), "Ma file does not exist")){
			return null;
		}	
		
		param[0] = "-m\"" + checkExtention(textMa,".ma") + "\"";
		param[1] = "-c" + coupleCombo.getText().trim();


		if(textAdv.isEnabled()){

			if(isEmpty("advanced value", textAdv))
				return null;
			param[3] = textAdv.getText().trim();
		}
		
		if (checkBoxs0.getSelection() == true){
			param[4] ="-s0"; 
			
		}else if (checkBoxs1.getSelection() == true){
			if(isEmpty("n value", nValue))
				return null;
			param[4] ="-s1-"+nValue.getText().trim(); 
			
		}else if (checkBoxs2.getSelection() == true){
			if(isEmpty("n value", nValue))
				return null;
			param[4] ="-s2-"+nValue.getText().trim(); 
			
		}else if (checkBoxs3.getSelection() == true){
			if(isEmpty("n value", nValue))
				return null;
			param[4] ="-s3-"+nValue.getText().trim(); 
			
		}else{
			MessageBox mbox = new MessageBox(new Shell(), SWT.ICON_ERROR | SWT.OK);
			mbox.setText("Error");
			mbox.setMessage("You MUST select a state type");
			mbox.open();
					
			return null;
		}		

		
		String combine = "";
		for (int n = 0;  n < 5; n++ ) {
			if (param[n] != null)
				combine = combine + " " + param[n];
		}
		combine = combine.concat( " " );
		return combine;
	}
	
	
	
	
	
	
	
	
	
	
	
	
	/*
	 * Performs proper action based on the botton pressed(Close or Proceed)
	 * 
	 * @param int buttonId
	 */
	protected void buttonPressed(int buttonId) {


		if (buttonId == IDialogConstants.CLOSE_ID){
			OK2Close = true;
			close();
		}
		if (buttonId == IDialogConstants.PROCEED_ID) {

			
			String command = collectParameters();
			if (command == null){
				doneButton.setEnabled(true);
				killButton.setEnabled(false);
				simuButton.setEnabled(true);
				return;
			}
			
			System.out.println(command);
			rsimu = new RunSimu(command,this);
			rsimu.start();
		} 
		super.buttonPressed(buttonId);
	}



	protected void configureShell(Shell shell) {
		super.configureShell(shell);
		//the following line registers the shell of SimuGui to parent
		//which causes the file-dialog box modal to the SimuGui dialog
		//this fixes the problem associated with the file-dialog not being modal to SimuGui 
		parent = shell;
		if (title != null)
			shell.setText(title);
	}




	protected void createButtonsForButtonBar(Composite parent) {
		// create OK and Cancel buttons by default
		simuButton = createButton(parent, IDialogConstants.PROCEED_ID, IDialogConstants.PROCEED_LABEL, true);
		simuButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e){
				killButton.setEnabled(true);
				simuButton.setEnabled(false);
				doneButton.setEnabled(false);
			}
		});
		doneButton = createButton(parent, IDialogConstants.CLOSE_ID, IDialogConstants.CLOSE_LABEL, true);
		killButton = createButton(parent, IDialogConstants.STOP_ID, "Kill", true);
		killButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e){
				simuProcess.destroy();
				try {
					Process lookProcess = Runtime.getRuntime().exec("ps");
					BufferedReader bf = new BufferedReader(new InputStreamReader(lookProcess.getInputStream()));
					String msg = null;
					while((msg = bf.readLine())!= null){
						if (msg.indexOf(path.toString().substring(2))!= -1){
							msg = msg.substring(2).trim();
							Process killProcess = Runtime.getRuntime().exec("kill -9 " + msg.substring(0,msg.indexOf(" ")));
						}
					}
				} catch (IOException e1) {
					e1.printStackTrace();
				}
				doneButton.setEnabled(true);
				killButton.setEnabled(false);
				simuButton.setEnabled(true);
				simuKilled = true;
			}
		});
		
		killButton.setEnabled(false);
		//do this here because setting the text will set enablement on the ok button
		textMa.setFocus();
	
	}
	
	
	
	/*
	 * This is the layout of the GUI which will appear when the button is pressed in the 
	 * tool bar
	 * 
	 * @param Composite parent
	 */	
	
	protected Control createDialogArea(Composite parent) {
		// create composite( which is a control that can contain other controls!)
		composite = (Composite)super.createDialogArea(parent);
		GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 10;
		composite.setLayout(gridLayout);

        //Row One - Should be text description of ma text box
		Label label = createLabel(parent, composite, message1);

		// Row two - Text Box for the Ma file and file button
		
		/*this section will look throught the project directory and check if the model file exist
		* If it does exists places the names of the cellular models found in the file into the 
		* cellular model text box (coupleCombo)
		*/
		textMa = new Text(composite, SWT.SINGLE | SWT.BORDER);
		if (maFileName != null){
			textMa.setText(maFileName);
		}
		textMa.addModifyListener( new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				coupleCombo.removeAll();
				boolean modelExist = false;
				IWorkspaceRoot root = (IWorkspaceRoot)PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().getInput();
				
				try {
					IResource[] files;
					IFile dropFile = null;
					files = root.getContainerForLocation(path).members();
					for(int i=0;i<files.length;i++){
						if(files[i]instanceof IFile && files[i].getName().equalsIgnoreCase( checkExtention(textMa,".ma") )){
							dropFile=(IFile) files[i];
							break;
						}
					}
					if (dropFile == null){
						modelExist=false;
					}else{
						modelExist=true;
					}
				
					if(modelExist){
										
						InputStream is = dropFile.getContents();
						BufferedReader br 	= new BufferedReader(new InputStreamReader(is));
						String line,couple = null;
						while ((line=br.readLine()) != null) {
	
							if(line.startsWith("["))
								couple=line.substring(1,line.lastIndexOf("]"));
							if (line.replaceAll(" ","").equalsIgnoreCase("type:cell"))
								coupleCombo.add(couple);		
						}
						if (coupleCombo.getItemCount()>0)
							coupleCombo.setText(coupleCombo.getItem(0));
					}
				} catch (Exception e1) {
					e1.printStackTrace();
				}	
			}
		});//end of listener
		
		Color c = new Color(null,255,255,255);
		textMa.setBackground(c);
		GridData gridData = new GridData();
		gridData.horizontalSpan=9;
		gridData.widthHint = convertHorizontalDLUsToPixels(IDialogConstants.MINIMUM_MESSAGE_AREA_WIDTH);
		textMa.setLayoutData(gridData);
		Button fileButtonMa = new Button(composite, SWT.PUSH);
		fileButtonMa.setText("Browse");
		fileButtonMa.setToolTipText("Select Ma File");
		GridData gridDataFBMa = new GridData();
		gridDataFBMa.horizontalAlignment = GridData.BEGINNING;
		fileButtonMa.setLayoutData(gridDataFBMa);
		fileButtonMa.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e){
			composite.setEnabled(false);
			simuButton.setEnabled(false);
			doneButton.setEnabled(false);
			saveBatButton.setEnabled(false);
			loadBatButton.setEnabled(false);
			OK2Close = false;
			String fileName = NewFileGui("*.Ma");
			if (!fileName.equals(""))
			textMa.setText( fileName);
			}
		});


		//Row Seven - A text description of the couple label
		label = createLabel(parent, composite, coupleMessage);
		
		//Row Eight - Text Drop Box of the couple Loader
		gridData = new GridData();
		coupleCombo = new Combo (composite, SWT.READ_ONLY);
		gridData.horizontalSpan =9;
		gridData.horizontalAlignment = GridData.FILL;
		gridData.widthHint = convertHorizontalDLUsToPixels(IDialogConstants.MINIMUM_MESSAGE_AREA_WIDTH);
		coupleCombo.setLayoutData(gridData);

		//Row Seven - A text description of the couple label
		label = createLabel(parent, composite, "Select initial state type:");

		//ME label = createLabel(parent, composite, "s0 [Use the values 0 and 1 (Uniform Distribution)]");
		//label = createLabel(parent, composite, "Fill cells with value {0 or 1} according to a Uniform Distribution. [n value not used]"); //ME
		
		/* //ORIGINAL SECTION START
		checkBoxs0 = createCheckBoxButton(composite, false);
		
		checkBoxs0.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e){
				if(checkBoxs0.getSelection() == true){
					checkBoxs1.setEnabled(false);
					checkBoxs2.setEnabled(false);
					checkBoxs3.setEnabled(false);
					nValue.setEnabled(false);					
				}else if(checkBoxs0.getSelection() == false){
					checkBoxs1.setEnabled(true);
					checkBoxs2.setEnabled(true);
					checkBoxs3.setEnabled(true);
					nValue.setEnabled(true);				
				}
			}
		});
		*/ //ORIGINAL SECTION END
		
		//ME REVISED SECTION START
		checkBoxs0 = createRadioButton(composite, false); 
		label = createLabel(parent, composite, "Fill cells with value {0 or 1} according to a Uniform Distribution. [n value not used]", 9); //ME moved
		
		checkBoxs0.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e){
				if(checkBoxs0.getSelection() == true){
					//checkBoxs1.setEnabled(false);
					//checkBoxs2.setEnabled(false);
					//checkBoxs3.setEnabled(false);
					nValue.setEnabled(false);					
				}else if(checkBoxs0.getSelection() == false){
					//checkBoxs1.setEnabled(true);
					//checkBoxs2.setEnabled(true);
					//checkBoxs3.setEnabled(true);
					nValue.setEnabled(true);				
				}
			}
		});		
		//ME REVISED SECTION END
		
	
		//ME label = createLabel(parent, composite, "s1-n [Use the value 1 for n cells & 0 for the rest]");
		//label = createLabel(parent, composite, "Fill n number of cells with {1}.  Fill rest of cells with {0}."); //ME

		/* //ORIGINAL SECTION START
		checkBoxs1 = createCheckBoxButton(composite, false);

		checkBoxs1.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e){
				if(checkBoxs1.getSelection() == true){

					checkBoxs0.setEnabled(false);
					checkBoxs2.setEnabled(false);
					checkBoxs3.setEnabled(false);
					
				}else if(checkBoxs0.getSelection() == false){

					checkBoxs0.setEnabled(true);
					checkBoxs2.setEnabled(true);
					checkBoxs3.setEnabled(true);
				
				}
			}
		});
		*/ //ORIGINAL SECTION END		

		//ME REVISED SECTION START
		checkBoxs1 = createRadioButton(composite, false);
		label = createLabel(parent, composite, "Fill n number of cells with {1}.  Fill rest of cells with {0}.", 9); //ME moved

		checkBoxs1.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e){
				if(checkBoxs1.getSelection() == true){
					nValue.setEnabled(true);
				}else if(checkBoxs0.getSelection() == false){
					nValue.setEnabled(false);
				}
			}
		});		
		//ME REVISED SECTION END
		
		// label = createLabel(parent, composite, "s2-n [Makes random states for the Pinball Model]");
		//label = createLabel(parent, composite, "Pinball Model: Fill n number of cells with {9}.  Fill one cell with {6}.  Fill rest of cells with {0}."); //ME

		/* //ORIGINAL SECTION START
		checkBoxs2 = createCheckBoxButton(composite, false);

		checkBoxs2.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e){
				if(checkBoxs2.getSelection() == true){

					checkBoxs0.setEnabled(false);
					checkBoxs1.setEnabled(false);
					checkBoxs3.setEnabled(false);
					
				}else if(checkBoxs0.getSelection() == false){

					checkBoxs0.setEnabled(true);
					checkBoxs1.setEnabled(true);
					checkBoxs3.setEnabled(true);
				
				}
			}
		});
		*/ //ORIGINAL SECTION END
		
		//ME REVISED SECTION START
		checkBoxs2 = createRadioButton(composite, false);
		label = createLabel(parent, composite, "Pinball Model: Fill n number of cells with {9}.  Fill one cell with {6}.  Fill rest of cells with {0}.", 9); //ME moved

		checkBoxs2.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e){
				if(checkBoxs2.getSelection() == true){
					nValue.setEnabled(true);
				}else if(checkBoxs0.getSelection() == false){
					nValue.setEnabled(false);
				}
			}
		});
		//ME REVISED SECTION END
	
		//ME label = createLabel(parent, composite, "s3-n [Random states for the Gas Dispersion Model]");
		//label = createLabel(parent, composite, "Gas Dispersion Model: Fill n number of cells with non-zero value.  Fill rest of cells with {0}."); //ME	

		/* //ORIGINAL SECTION START
		checkBoxs3 = createCheckBoxButton(composite, false);

		checkBoxs3.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e){
				if(checkBoxs3.getSelection() == true){

					checkBoxs0.setEnabled(false);
					checkBoxs1.setEnabled(false);
					checkBoxs2.setEnabled(false);
					
				}else if(checkBoxs0.getSelection() == false){

					checkBoxs0.setEnabled(true);
					checkBoxs1.setEnabled(true);
					checkBoxs2.setEnabled(true);
				
				}
			}
		});
		*/ //ORIGINAL SECTION END
		
		//ME REVISED SECTION START
		checkBoxs3 = createRadioButton(composite, false);
		label = createLabel(parent, composite, "Gas Dispersion Model: Fill n number of cells with non-zero value.  Fill rest of cells with {0}.", 9); //ME moved

		checkBoxs3.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e){
				if(checkBoxs3.getSelection() == true){
					nValue.setEnabled(true);
				}else if(checkBoxs0.getSelection() == false){
					nValue.setEnabled(false);
				}
			}
		});
		//ME REVISED SECTION END


		//Row Five - Text Description of the Out text box
		//ME label = createLabel(parent, composite, "n value");
		label = createLabel(parent, composite, "Enter n value (must be integer):"); //ME

		nValue= new Text(composite, SWT.SINGLE | SWT.BORDER);
		GridData gridData9 = new GridData();
		//nValue.setTextLimit(50);
		gridData9.horizontalAlignment = GridData.FILL;
		gridData9.horizontalSpan = 10;
		nValue.setLayoutData(gridData9);
		
		nValue.setEnabled(true);
		nValue.setToolTipText("n value:");
		nValue.setText("0");
		nValue.addKeyListener(new KeyListener(){
			public void keyPressed(KeyEvent e) {
				
				if (!nValue.getText().trim().equals("-")){// this is so the user can enter (-)ve numbers 
				
					try{
						Integer value= new Integer(nValue.getText());
						if(e.keyCode == SWT.ARROW_UP){
							value = new Integer(value.intValue()+1);
							nValue.setText(value.toString());
						}else if(e.keyCode == SWT.ARROW_DOWN){
							value = new Integer(value.intValue()-1);
							nValue.setText(value.toString());
						}
	
					}catch(NumberFormatException e2){
						nValue.setText("");    //clear textbox if error
					}catch(Exception e1){
						e1.printStackTrace();
					}
				}
			}

			public void keyReleased(KeyEvent e) {
						
			}
		});	


		//Row Thirteen - Text Discription of Advanced					
		label = createLabel(parent, composite, message4);

		//Row Fourteen - The check box and text box for advanced		
		checkBoxAdv = createCheckBoxButton(composite, false);
		
		checkBoxAdv.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e){
				if(textAdv.getEnabled() == true) { 
					textAdv.setEnabled(false);
					checkedAdv = false;
				}	
				else { 
					textAdv.setEnabled(true);
					checkedAdv = true;
				}
			}
		});
		
		textAdv = new Text(composite, SWT.SINGLE | SWT.BORDER);
		GridData gridData18 = new GridData();
		gridData18.horizontalAlignment = GridData.FILL;
		gridData18.grabExcessHorizontalSpace = true;
		gridData18.horizontalSpan = 9;
		textAdv.setLayoutData(gridData18);
		textAdv.setEnabled(checkedAdv);

//		Row fifteen - Text description of the Comment text box
		Label label8 = createLabel(parent, composite, message5);
		
//		Row Sixteen - The check box and text box for comment
		checkBoxComment = createCheckBoxButton(composite, false);
		
		checkBoxComment.addSelectionListener(new SelectionAdapter() {
			 public void widgetSelected(SelectionEvent e){
				  if(textComment.getEnabled() == true) { 
					  textComment.setEnabled(false);
					  checkedComment = false;	
				  }
				  else { 
					  textComment.setEnabled(true);
					  checkedComment = true;
				  }
			  }
		});			
		textComment= new Text(composite, SWT.SINGLE | SWT.BORDER);
		GridData gridData21 = new GridData();
		gridData21.horizontalSpan = 9;
		gridData21.horizontalAlignment = GridData.FILL;
		gridData21.grabExcessHorizontalSpace = true;
		textComment.setLayoutData(gridData21);
		textComment.setEnabled(false);
	  
//		Row Seventeen - Save as Batch button
	    saveBatButton = new Button(composite, SWT.PUSH);
		saveBatButton.setText("Save as .bat");
		saveBatButton.setToolTipText("Save Settings to batch file");
		GridData gridDataSBB= new GridData();
		gridDataSBB.horizontalAlignment = GridData.END;
		gridDataSBB.horizontalSpan = 9;	
		saveBatButton.setLayoutData(gridDataSBB);
		saveBatButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e){
				 composite.setEnabled(false);
				 simuButton.setEnabled(false);
				 doneButton.setEnabled(false);
				 saveBatButton.setEnabled(false);
				 loadBatButton.setEnabled(false);
				 OK2Close = false;
				 SaveBatchFileGui("*.bat");
			}
		});		
		
	
//		Row Eleven - Load Batch button
		loadBatButton = new Button(composite, SWT.PUSH);
		loadBatButton.setText("Load .bat");
		loadBatButton.setToolTipText("Load Batch from file");
		GridData gridDataLBB = new GridData();
		gridDataLBB.horizontalAlignment = GridData.BEGINNING;
		loadBatButton.setLayoutData(gridDataLBB);
		loadBatButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e){
				try {
				
					BufferedReader batchReader = new BufferedReader(new FileReader(path.toString() + NewFileGui("*.bat")));
					String command = null;
					command = batchReader.readLine();
					System.out.println("\nloading: " + command);
			  
					try{
									
						String comments = batchReader.readLine();
						while(!comments.trim().startsWith("rem"))
							comments = batchReader.readLine();
						textComment.setText(comments.substring(4));
						checkedComment = true;
						checkBoxComment.setSelection(checkedComment);
						textComment.setEnabled(checkedComment);
					}catch(NullPointerException e1){
						e1.printStackTrace();
					}
		
					int indexStartMa = command.indexOf( "-m" );
				
					int indexEndMa = command.indexOf( ".ma" );
					if(indexEndMa ==-1)
						indexEndMa = command.indexOf( ".MA" );
					
					int indexStartCell = command.indexOf( "-c" );
					int indexEndCell = command.indexOf( " ", indexStartCell );

					int indexStartAdv = command.indexOf( " ", indexStartCell ); //ME

					int sValueStart =  command.indexOf( "-s" );					
					int sValueEnd = command.indexOf( " ", sValueStart );
									
					if (indexStartMa != -1){
						textMa.setText( command.substring(indexStartMa+2, indexEndMa+3 ).trim() ); 
					}else{
						textMa.setText( "" ); 
					}
					
					if (indexStartCell != -1) {		
						coupleCombo.setText( command.substring(indexStartCell + 2, indexEndCell ).trim() );						
					}
					
					/* //ORIGINAL SECTION START
					if ( sValueStart != -1){
						if ( command.indexOf( "-s0" )!= -1){
							
						
						}else if ( command.indexOf( "-", sValueStart )!= -1 ){
							nValue.setText( command.substring(sValueStart + 4, sValueEnd ).trim() );
							if (command.indexOf( "-s1" ) != -1){
								checkBoxs1.setSelection(true);
								checkBoxs0.setEnabled(false);
								checkBoxs2.setEnabled(false);
								checkBoxs3.setEnabled(false);
							
							}else if (command.indexOf( "-s2" ) != -1){
								checkBoxs2.setSelection(true);
								checkBoxs0.setEnabled(false);
								checkBoxs1.setEnabled(false);
								checkBoxs3.setEnabled(false);
							
							}else if (command.indexOf( "-s3" ) != -1){
								checkBoxs3.setSelection(true);
								checkBoxs0.setEnabled(false);
								checkBoxs1.setEnabled(false);
								checkBoxs2.setEnabled(false);
							
							}
						}					
					}
					*/ //ORIGINAL SECTION END
					
					//ME REVISED SECTION START
					if ( sValueStart != -1){
						if ( command.indexOf( "-s0" )!= -1){
							checkBoxs0.setSelection(true);
							checkBoxs1.setSelection(false);
							checkBoxs2.setSelection(false);
							checkBoxs3.setSelection(false);			
							nValue.setText("");
							nValue.setEnabled(false);				
						
						}else if ( command.indexOf( "-", sValueStart )!= -1 ){
							nValue.setEnabled(true);
							nValue.setText( command.substring(sValueStart + 4, sValueEnd ).trim() );

							if (command.indexOf( "-s1" ) != -1){
								checkBoxs0.setSelection(false);
								checkBoxs1.setSelection(true);
								checkBoxs2.setSelection(false);
								checkBoxs3.setSelection(false);								
							}else if (command.indexOf( "-s2" ) != -1){
								checkBoxs0.setSelection(false);
								checkBoxs1.setSelection(false);
								checkBoxs2.setSelection(true);
								checkBoxs3.setSelection(false);							
							}else if (command.indexOf( "-s3" ) != -1){
								checkBoxs0.setSelection(false);
								checkBoxs1.setSelection(false);
								checkBoxs2.setSelection(false);
								checkBoxs3.setSelection(true);								
							}
						}					
					}
					//ME REVISED SECTION END

					System.out.println("s  " + sValueEnd);
					
					/* //ORIGINAL SECTION START
					if (sValueEnd+1 < command.length()){
						textAdv.setEnabled(true);
						checkBoxAdv.setSelection(true);
						textAdv.setText(command.substring(sValueEnd, command.length() ).trim());
					}else {
						checkBoxAdv.setSelection(false);
						textAdv.setEnabled(false);
						System.out.println("false dfdfdsfa");
					}
					*/ //ORIGINAL SECTION END
					
					//ME REVISED SECTION START
					if ( (indexStartAdv!=-1) && ( (sValueStart - indexStartAdv) > 1) ){
						//advanced present
						checkBoxAdv.setSelection(true);
						textAdv.setEnabled(true);
						textAdv.setText(command.substring(indexStartAdv + 1, sValueStart-1) );
					} else {
						//no advanced 
						textAdv.setText("");
						checkBoxAdv.setSelection(false);
						textAdv.setEnabled(false);	
					}					
					//ME REVISED SECTION END

				}catch (Exception error) {
					  error.printStackTrace();
					  System.out.println("************EXCEPTION: loading batch file******");
				}
			 }
		});
	
		Label errorMessageLabel = new Label(composite, SWT.NONE);
		errorMessageLabel.setFont(parent.getFont());

		return composite;
	}
	
	
	/*
	 *creats a checkBox for dialog box
	 *
	 * @param Composite composite 
	 * @param boolean used to check off the box
	 * 
	 * @return Button 
	 **/	
	
	
	private Button createCheckBoxButton(Composite composite, boolean checked) {
		Button checkBox = new Button(composite, SWT.CHECK);
		GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.BEGINNING;
		checkBox.setLayoutData(gridData);
		checkBox.setSelection(checked);
		return checkBox;
	}
	
		
	//ME REVISED SECTION START
	private Button createRadioButton(Composite composite, boolean checked) {
		Button radioB = new Button(composite, SWT.RADIO);
		GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.END;
		radioB.setLayoutData(gridData);
		radioB.setSelection(checked);
		return radioB;
	}
	//ME REVISED SECTION END
	
	
	
	/*
	 *creats a lable for dialog box
	 *
	 * @param String message
	 * @param Composite composite 
	 * @param Composite parent 
	 * 
	 * @return Label 
	 **/
	
	private Label createLabel(Composite parent, Composite composite, String message) {
		Label label = new Label(composite, SWT.NONE);
		label.setText(message);
		GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.horizontalSpan=10;		
		label.setLayoutData(gridData);
		label.setFont(parent.getFont());
		return label;
	}	
	
	private Label createLabel(Composite parent, Composite composite, String message, int size) {
		Label label = new Label(composite, SWT.NONE);
		label.setText(message);
		GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.horizontalSpan=size;		
		label.setLayoutData(gridData);
		label.setFont(parent.getFont());
		return label;
	}	

	/**
		 * @see org.eclipse.jface.window.Window#close()
		 */
	public boolean close() {
		
		if (OK2Close) {
							
			boolean returnValue = super.close();
			return returnValue;
		}
		
		return true;
		
	}
	
	/**
	 * This method takes the fileName and concatenates the digits of n 
	 * [found through testString] to the end of the string [if required].
	 * Edited: by Jing Cao
	 * 
	 * @param fileName
	 * @param integer to be appended
	 * @return String tempFileName [new file name with the number at end]
	 */
	private String setDigit(String fileName, int n) { //beg setDigit
		String tempFileName = new String(fileName);
		String tempString;
		char c;
		int i;

		//Note: the string array only extends from 0 -> (length of string-1)
		for (i = (fileName.length() - 1); i >= 0; i--) { //This for loop will find the location where the number starts.
			c = tempFileName.charAt(i);
			if ((c < '0') || (c > '9'))
				//if the chracter is not a number then break out of the for loop
				break;
			// the value of 'i' will be the index where there is a character [not a number]
		}

		tempFileName = tempFileName.trim();
		//redundant, it gets rid of spaces infront or after the fileName
		tempString = new String(Integer.toString(n));
		if (i < 0) //the fileName consists of all numbers
			return tempString;
		// then return the updated number as the file name!
		//else
		tempFileName = tempFileName.substring(0, i + 1);
		//get the character part of the string
		//note: the +1 is required because substring(beg,end) is "exclusive" of the end.

		return tempFileName.concat(tempString);
		//adds the updated number to the end of the characters and returns the string
	} //end setDigit

	/**
	 * This method takes a file name (stripped off ".bat") and
	 * returns the digits (numbers) at the end as a string.
	 * Returns '0' if there's no number at the end of the file name.
	 * Will return the file name if the string is composed of only numbers.
	 * Edited by Jing Cao.
	 * 
	 * @param fileName
	 * @return number [of the type string]
	 */
	private String testString(String fileName) { //beg testString()

		//Variable Declarations
		String tempString = new String(fileName);
		//redunant but just to make shure that we don't make changes to testString()
		String number = new String(""); //this string stores the number
		int i;
		char c;

		//Note: the string array only extends from 0 -> (length of string-1)
		for (i = (fileName.length() - 1); i >= 0; i--) { //This for loop will find the location where the number starts.
			c = tempString.charAt(i);
			if ((c < '0') || (c > '9'))
				//if the chracter is not a number then break out of the for loop
				break;
			// the value of 'i' will be the index where there is a character [not a number]
		}

		//if statements to set the number string properly
		if (i == (fileName.length() - 1))
			//sets number to 0 if no number found at end of the string.
			number = "0";
		else if (i == -1) //means that the file name is a number!
			number = new String(fileName);
		//then set number to be the file name
		else
			number = tempString.substring(i + 1);
		//creates the number string by chopping off the letters

		number = number.trim();
		//removes any spaces at the beginning or the end of the string

		return number;
	} //end testString()
	
	//this function is used by load.bat to load a new file. 
	private String NewFileGui(String filter) {
		
		composite.setEnabled(false);
		simuButton.setEnabled(false);
		doneButton.setEnabled(false);
		OK2Close = false;
		
		FileDialog fileDialog = new FileDialog(parent);
		String[] extention = new String[1];
		extention[0] = filter;
		fileDialog.setFilterPath(path.toString());
		fileDialog.setFilterExtensions(extention);
		fileDialog.open();
		
		composite.setEnabled(true);
		simuButton.setEnabled(true);
		doneButton.setEnabled(true);
		loadBatButton.setEnabled(true);
		saveBatButton.setEnabled(true);
		OK2Close = true;
		return fileDialog.getFileName();
	}
	
	
	//this function is used by save.bat to save a file
	//code modified by Sameen Rehman
	private void SaveBatchFileGui(String filter) {
		String[] param = new String[10];
		
		String command = collectParameters();
	    String temp;
		if (command == null){
		
			composite.setEnabled(true);
			doneButton.setEnabled(true);
			simuButton.setEnabled(true);
			saveBatButton.setEnabled(true);
			loadBatButton.setEnabled(true);
			OK2Close = true;
					
			return;
		}
		
		composite.setEnabled(false);
		simuButton.setEnabled(false);
		doneButton.setEnabled(false);
				
		FileDialog fileDialog = new FileDialog(parent, SWT.SAVE);
		String[] extention = new String[1];
		extention[0] = filter;
		fileDialog.setFilterPath(path.toString());
		fileDialog.setFilterExtensions(extention);
		fileDialog.open();
		
		String fileName = fileDialog.getFileName();
		
		try {
			
			if (fileName.endsWith(".bat")) {
				fileName = fileName.substring(0, fileName.length()-4);
			}
			//this code segment makes sure nothing is saved when u cancel the save dialog box
			if(fileName == ""){
			fileDialog.setFilterPath("");
			fileDialog.setFilterExtensions(null);
			composite.setEnabled(true);
			simuButton.setEnabled(true);
			doneButton.setEnabled(true);
			saveBatButton.setEnabled(true);
			loadBatButton.setEnabled(true);
			
			OK2Close = true;
			return;
			}
			
			File existFile = new File(path.toString() + fileName + ".bat");
			boolean cancelled = false;
			if (existFile.exists()) {
				
				IPreferenceStore store = CDBuilderPlugin.getDefault().getPreferenceStore();
				if(store.getInt("batchChoice") == 3) {
					OverwriteGui ovG = new OverwriteGui(new Shell());
					ovG.open();
					if (ovG.buttonPressed() == 0) {
						System.out.print("canceled saving bat");	
						cancelled = true;
						
						composite.setEnabled(true);
						simuButton.setEnabled(true);
						doneButton.setEnabled(true);
						saveBatButton.setEnabled(true);
						loadBatButton.setEnabled(true);
						OK2Close = true;
					}
					if (ovG.buttonPressed() == 2) {
						boolean looking = true;
						
						while(looking) {
							n=(Integer.decode(testString(fileName))).intValue();
							n++;
							
							temp = setDigit(fileName,n);
							File existFileCount = new File(path.toString() + temp+ ".bat");
							System.out.println(path.toString() + temp + ".bat");
							fileName = temp;
							if(!existFileCount.exists()) {
								looking = false;
																	
								MessageBox mbox = new MessageBox(new Shell(),SWT.ICON_INFORMATION | SWT.OK);
								mbox.setText("Saving File");
								mbox.setMessage("The file has been saved as "+ fileName+".bat");
								mbox.open();
																
								composite.setEnabled(true);
								simuButton.setEnabled(true);
								doneButton.setEnabled(true);
								saveBatButton.setEnabled(true);
							    loadBatButton.setEnabled(true);
							}
						}
						if(ovG.noAskAgain() )
							store.setValue("batchChoice", 2);
					}
					if (ovG.buttonPressed() == 1) {
						if(ovG.noAskAgain() )
							store.setValue("batchChoice", 1);
					}
				
				
				}
				else if(store.getInt("batchChoice") == 2) {
					boolean looking = true;
					int n = 0;
					while(looking) {
						n++;
						File existFileCount = new File(path.toString() + fileName + "-" + n + ".bat");
						System.out.println(path.toString() + fileName +"-" + n + ".bat");
						if(!existFileCount.exists()) {
							looking = false;
							fileName = fileName.concat("-" + Integer.toString(n));
						}
					}
				}
				else {
				}
			}
			if (!cancelled) {	
				composite.setEnabled(false);
				simuButton.setEnabled(false);
				doneButton.setEnabled(false);
				OK2Close = false;
				FileWriter batchFile = new FileWriter(path.toString() + fileName + ".bat");
				System.out.println(path.toString() + fileName + ".bat");
				param[0] = "-m" +textMa.getText();
				

				System.out.println(command);
				batchFile.write(command + "\n");
				batchFile.write("rem " + textComment.getText() + "\n");
				batchFile.write("pause");
				batchFile.close();
				composite.setEnabled(true);
				simuButton.setEnabled(true);
				doneButton.setEnabled(true);
				saveBatButton.setEnabled(true);
				loadBatButton.setEnabled(true);
				OK2Close = true;
			}
		}
		catch (Exception e) {
			e.printStackTrace();
		}	
	}
	
}
